# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'start_window.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QHBoxLayout, QLabel,
    QLineEdit, QMainWindow, QMenuBar, QPushButton,
    QSizePolicy, QSpacerItem, QStatusBar, QVBoxLayout,
    QWidget)

from debyetools.tpropsgui.custom_widgets import ClickableLineEdit

class Ui_StartWindow(object):
    def setupUi(self, StartWindow):
        if not StartWindow.objectName():
            StartWindow.setObjectName(u"StartWindow")
        StartWindow.resize(373, 181)
        self.centralwidget = QWidget(StartWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.verticalLayout_2 = QVBoxLayout(self.centralwidget)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.label = QLabel(self.centralwidget)
        self.label.setObjectName(u"label")

        self.horizontalLayout.addWidget(self.label)

        self.lineEdit_compoundname = ClickableLineEdit(self.centralwidget)
        self.lineEdit_compoundname.setObjectName(u"lineEdit_compoundname")
        self.lineEdit_compoundname.setReadOnly(True)

        self.horizontalLayout.addWidget(self.lineEdit_compoundname)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_2)

        self.label_3 = QLabel(self.centralwidget)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setEnabled(False)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)

        self.horizontalLayout.addWidget(self.label_3)

        self.lineEdit_mass = ClickableLineEdit(self.centralwidget)
        self.lineEdit_mass.setObjectName(u"lineEdit_mass")
        self.lineEdit_mass.setEnabled(True)
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.lineEdit_mass.sizePolicy().hasHeightForWidth())
        self.lineEdit_mass.setSizePolicy(sizePolicy1)
        self.lineEdit_mass.setMinimumSize(QSize(100, 0))
        self.lineEdit_mass.setReadOnly(True)

        self.horizontalLayout.addWidget(self.lineEdit_mass)


        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.label_2 = QLabel(self.centralwidget)
        self.label_2.setObjectName(u"label_2")

        self.horizontalLayout_2.addWidget(self.label_2)

        self.lineEdit_2 = QLineEdit(self.centralwidget)
        self.lineEdit_2.setObjectName(u"lineEdit_2")

        self.horizontalLayout_2.addWidget(self.lineEdit_2)

        self.horizontalSpacer_3 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer_3)


        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.checkBox = QCheckBox(self.centralwidget)
        self.checkBox.setObjectName(u"checkBox")

        self.horizontalLayout_3.addWidget(self.checkBox)


        self.verticalLayout_2.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_4.addItem(self.horizontalSpacer)

        self.pushButtonNext = QPushButton(self.centralwidget)
        self.pushButtonNext.setObjectName(u"pushButtonNext")

        self.horizontalLayout_4.addWidget(self.pushButtonNext)


        self.verticalLayout_2.addLayout(self.horizontalLayout_4)

        StartWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(StartWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 373, 21))
        StartWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(StartWindow)
        self.statusbar.setObjectName(u"statusbar")
        StartWindow.setStatusBar(self.statusbar)

        self.retranslateUi(StartWindow)

        QMetaObject.connectSlotsByName(StartWindow)
    # setupUi

    def retranslateUi(self, StartWindow):
        StartWindow.setWindowTitle(QCoreApplication.translate("StartWindow", u"debyetools - [compound]", None))
        self.label.setText(QCoreApplication.translate("StartWindow", u"Compound: ", None))
        self.lineEdit_compoundname.setText("")
        self.label_3.setText(QCoreApplication.translate("StartWindow", u"mass [kg/mol-at]", None))
        self.lineEdit_mass.setText("")
        self.label_2.setText(QCoreApplication.translate("StartWindow", u"Phase name: ", None))
        self.checkBox.setText(QCoreApplication.translate("StartWindow", u"Specify structure (for interatomic potential)", None))
        self.pushButtonNext.setText(QCoreApplication.translate("StartWindow", u"Next", None))
    # retranslateUi




import numpy as np
import debyetools.potentials as potentials
from debyetools.aux_functions import load_V_E
V_data, E_data = load_V_E('../Al_fcc/SUMMARY', '../Al_fcc/CONTCAR')
V_data, E_data = V_data*(1E-30*6.02E+23), E_data*(1.60218E-19 * 6.02214E+23)
params_initial_guess = [-3e5, 1e-5, 7e10, 4]
Birch_Murnaghan = potentials.BM()
Birch_Murnaghan.fitEOS(V_data, E_data, params_initial_guess)
print(Birch_Murnaghan.pEOS)



from PySide6.QtWidgets import QDialog, QMessageBox

from debyetools.tpropsgui.ui_dialog_fitEOS import Ui_Form as Ui_iparams

from debyetools.tpropsgui.atomtools import dt_potentials

import numpy as np

from debyetools.tpropsgui.dialog_loadEOS import dialogLoadEOS

from PySide6.QtCore import QTimer

from debyetools.tpropsgui.plot_EV import windowPlot
from PySide6.QtGui import QPixmap, QPalette


def highlight_line_edit(line_edit, color="purple", duration=100):
    # Set the background color
    line_edit.setStyleSheet(f"background-color: {color};")

    # Create a QTimer to reset the color after `duration` milliseconds
    timer = QTimer(line_edit)
    timer.setSingleShot(True)  # Only trigger once
    timer.timeout.connect(lambda: line_edit.setStyleSheet(""))  # Reset the color
    timer.start(duration)


def calculate_r_squared(y_obs, y_pred):
    """
    Calculate the coefficient of determination (R^2) between observed and predicted data.

    Parameters:
        y_obs (array-like): Observed data values.
        y_pred (array-like): Predicted data values.

    Returns:
        float: The R^2 value.
    """
    y_obs = np.array(y_obs)
    y_pred = np.array(y_pred)

    # Calculate SS_res and SS_tot
    ss_res = np.sum((y_obs - y_pred) ** 2)
    ss_tot = np.sum((y_obs - np.mean(y_obs)) ** 2)

    # Calculate R^2
    r_squared = 1 - (ss_res / ss_tot)

    return r_squared

class dialogFitEOS(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_iparams()
        self.ui.setupUi(self)

        self.ui.pushClose.clicked.connect(self.on_pushSave)
        self.ui.pushButton_2.clicked.connect(self.on_pushButton_fitEOS)
        self.ui.pushloadEvV.clicked.connect(self.on_pushloadEvV)

        self.dialog_loadEOS = dialogLoadEOS(self)

        # Connect the textChanged signals to a shared color change method
        self.ui.EvVText_2.textChanged.connect(lambda: self.on_text_changed(self.ui.EvVText_2))
        self.ui.lineEdit_3.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_3))

        self.plotwindow = windowPlot(self)
        self.ui.pushPlot.clicked.connect(self.plotEV)

    def get_EvV(self):
        if self.ui.radioButton_2.isChecked():
            conv = [(1e-30 * 6.02e23), (0.160218e-18 * 6.02214e23)]
        elif self.ui.radioButton_4.isChecked():
            conv = [1, 1]

        txt = self.ui.EvVText_2.toPlainText().split('\n')
        data_lst = []
        for ti in txt:
            if len(ti) == 0: continue
            if ti[0] == '#': continue
            data_lst.append([float(tii) for tii in ti.split()])
        data = np.array(data_lst)
        ncols = len(data.T)
        return (data[:, i] * conv[i] for i in range(ncols))

    def get_EOS_params(self):
        txt = self.ui.lineEdit_3.text()
        if txt == '':
            return -3e5, 9e-6, 7e10, 4
        return [float(ti) for ti in txt.replace(' ', '').split(',')]

    def on_pushloadEvV(self):
        self.dialog_loadEOS.mass = 0
        self.dialog_loadEOS.EvVtext = self.ui.EvVText_2
        self.dialog_loadEOS.show()


    def on_pushSave(self):
        current_text = self.ui.comboBox_2.currentText()
        self.external_combobox.setCurrentText(current_text)
        self.external_EOS_txt.setText(self.ui.lineEdit_3.text())
        self.close()

    def on_pushButton_fitEOS(self):
        dict_eos = {'Birch-Murnaghan': 'BM', 'Rose-Vinet': 'RV', 'Mie-Gruneisen': 'MG', 'TB-SMA': 'TB',
                    'Murnaghan': 'MU', 'Poirier-Tarantola': 'PT', 'Morse potential': 'MP',
                    'EAM int. potential': 'EAM'}
        current_text = self.ui.comboBox_2.currentText()

        self.eos_str = dict_eos[current_text]

        self.ui.progress_3.setValue(33)
        args = (None,)
        if self.eos_str == 'MP':
            print(self.molecule.__dir__())
            print(self.molecule_from_crystal.__dir__())
            self.molecule.formula, self.molecule.cell, self.molecule.basis, self.molecule.cutoff, self.molecule.number_of_NNs = self.molecule_from_crystal.formula, self.molecule_from_crystal.cell, self.molecule_from_crystal.basis, self.molecule_from_crystal.cutoff, self.molecule_from_crystal.number_of_NNs
            args = self.molecule.formula, self.molecule.cell, self.molecule.basis, self.molecule.cutoff, self.molecule.number_of_NNs
        self.eos = getattr(dt_potentials, self.eos_str)(*args)  # *self.ipotparamsdialog.args)

        try:
            Vdata, Edata = self.get_EvV()
        except Exception as e:
            print(e)
            error_msg = 'Something is wrong with the data for energy versus volume.\n Please check and try again.'
            QMessageBox.information(self, 'Error', error_msg, QMessageBox.Ok)


        self.Vdata = Vdata
        self.Edata = Edata

        initial_guess = self.get_EOS_params()
        self.eos.fitEOS(Vdata, Edata, initial_parameters=initial_guess, fit=True)

        self.ui.lineEdit_3.setText(', '.join(['%.9e' % (p) for p in self.eos.pEOS]))

        self.ui.progress_3.setValue(100)

    def is_dark_mode(self):
        # Detect if the application is in dark mode using the palette
        palette = self.palette()
        return palette.color(QPalette.ColorRole.Window).value() < 128  # Lightness threshold for dark mode

    def on_text_changed(self, line_edit):
        # Call the reusable highlight function
        dark_mode = self.is_dark_mode()
        color_p = "purple" if dark_mode else "#94a2c9"

        highlight_line_edit(line_edit, color=color_p, duration=500)
        # Update label with the current content of the edited line edit



    def plotEV(self):
        try:

            Xdata = self.Vdata/1e-5
            Ydata = self.Edata/1e3

            self.plotwindow.ax.cla()
            self.plotwindow.ax.plot(Xdata, Ydata, ls = '', marker='.', label = 'data', mfc='None', mec='black', markersize=10,mew=1)
            self.plotwindow.ax.plot(Xdata, [self.eos.E0(Vi)/1e3 for Vi in Xdata*1e-5], ls = '', marker='+', label = 'model', mfc='purple', mec='purple', markersize=10,mew=1)

            err = calculate_r_squared(Ydata, [self.eos.E0(Vi)/1e3 for Vi in Xdata*1e-5])
            self.plotwindow.ax.set_title(r'$R^2$ = ' + f'{err:.5f}')
            spanX = (max(Xdata) - min(Xdata))/20
            spanY = (max(Ydata) - min(Ydata))/20
            self.plotwindow.ax.set_xlim((min(Xdata)-spanX, max(Xdata)+spanX))
            self.plotwindow.ax.set_ylim((min(Ydata)-spanY, max(Ydata)+spanY))
            self.plotwindow.ax.legend(loc=9)
            self.plotwindow.ax.set_xlabel(r'$10^{-5}m^3/mol$-$at$')
            self.plotwindow.ax.set_ylabel(r'$kJ/mol$-$at$')

            self.plotwindow.show()
        except:
            # Create a message box
            msg_box = QMessageBox()
            msg_box.setWindowTitle("Warning")
            msg_box.setText("Data not entered yet.\nPlease run the fitting first.")

            # Add Yes and No buttons
            msg_box.setStandardButtons(QMessageBox.Ok)
            response = msg_box.exec_()



import numpy as np
from PySide6.QtCore import Qt, QTimer
from PySide6.QtWidgets import QMainWindow, QTableWidgetItem, QMenu, QMessageBox, QApplication
from debyetools.fs_compound_db import fit_FS as dt_fit_FS
from matplotlib.backends.backend_qt5agg import FigureCanvas
from matplotlib.figure import Figure
from matplotlib.widgets import Cursor

from debyetools.tpropsgui.atomtools import atom_energy
from debyetools.tpropsgui.ui_cp_window import Ui_MainWindow as Ui_Cp
from PySide6.QtGui import QPixmap, QPalette


# from debyetools.fs_compound_db import Cp2fit as dt_Cp2fit


def highlight_line_edit(line_edit, color="purple", duration=100):
    # Set the background color
    line_edit.setStyleSheet(f"background-color: {color};")

    # Create a QTimer to reset the color after `duration` milliseconds
    timer = QTimer(line_edit)
    timer.setSingleShot(True)  # Only trigger once
    timer.timeout.connect(lambda: line_edit.setStyleSheet(""))  # Reset the color
    timer.start(duration)


# Function to normalize data to range [0, 1]
def normalize(data):
    return (data - np.min(data)) / (np.max(data) - np.min(data))


class dialogCpWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_Cp()
        self.ui.setupUi(self)
        #        self.app = app

        self.fig = Figure(figsize=(3, 3))
        self.canvas = FigureCanvas(self.fig)
        self.set_canvas_configuration()

        llayout = self.ui.horizontalLayout  # QHBoxLayout()
        llayout.addWidget(self.canvas, 88)
        self.canvas.setContextMenuPolicy(Qt.CustomContextMenu)
        self.canvas.customContextMenuRequested.connect(self.right_menu)

        self.ui.pushButton.clicked.connect(self.on_click_recalc)
        self.ui.lineEdit.textChanged.connect(self.update_FS_Tto)
        self.ui.comboBox.currentIndexChanged.connect(self.selectionchange)

        self.ui.comboBox_2.currentIndexChanged.connect(self.selectionchange_plot)
        self.current_plot = 9

        self.ui.progress.setGeometry(700, 505, 81, 5)

        self.ui.pushBack.clicked.connect(self.on_pushBack)

        self.ui.pushExport.clicked.connect(self.on_pushExport)
        self.ui.pushCloseAll.clicked.connect(self.on_pushCloseAll)

        # Connect the textChanged signals to a shared color change method
        self.ui.lineEdit.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit))
        self.ui.lineEdit_2.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_2))
        self.ui.lineEdit_3.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_3))
        self.ui.lineEdit_4.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_4))
        self.ui.tableWidget.cellChanged.connect(lambda: self.on_text_changed(self.ui.tableWidget))


    def on_pushCloseAll(self):

        # Create a message box
       msg_box = QMessageBox()
       msg_box.setWindowTitle("Confirm Action")
       msg_box.setText("Do you want to proceed?")

       # Add Yes and No buttons
       msg_box.setStandardButtons(QMessageBox.Yes | QMessageBox.No)

       # Execute the message box and capture the response
       response = msg_box.exec_()

       # Check which button was clicked
       if response == QMessageBox.Yes:
           QApplication.instance().closeAllWindows()
           # Trigger Yes action
       elif response == QMessageBox.No:
           pass
           # Trigger No action

    def on_pushExport(self):
        txt4output = f'{self.formula}$'

        txt4output += f'{self.Ef:.7e}'

        txt4output += f'${self.S298:7e}$'

        lst4output = []
        rcount = self.ui.tableWidget.rowCount()
        for ix in range(2, rcount):
            p = self.ui.tableWidget.item(ix, 0).text()
            lst4output.append(f'{p}')
        txt4output += '&'.join(lst4output)
        txt4output += '$'
        t4out = [f'{p:.2e}' for p in [self.FS_Tfrom, self.FS_Tto]]
        txt4output += '&'.join(t4out)

        with open('export_dtoutput4cmpnd', 'w') as f:
            f.write(txt4output)

        QMessageBox.information(self, 'Parameters Saved', "Data was saved as \n'export_dtoutput4cmpnd' file.", QMessageBox.Ok)




    def selectionchange_plot(self, i):
        self.current_plot = i
        self.plot_prop('T', self.proplist[i])

    def selectionchange(self, i):
        Pi = self.Ps[i]
        self.ui.tableWidget.setItem(0, 0, QTableWidgetItem('%.5e' % (self.dict_H298['%.1f' % (Pi / 1e9)])))
        self.ui.tableWidget.setItem(1, 0, QTableWidgetItem('%.5e' % (self.dict_S298['%.1f' % (Pi / 1e9)])))
        for ix, p in enumerate(self.dict_FS['%.1f' % (Pi / 1e9)]['Cp']):
            self.ui.tableWidget.setItem(ix + 2, 0, QTableWidgetItem('%.5e' % (p)))

    def get_T(self):
        T_initial, T_final, Tstep = (float(sti) for sti in self.ui.lineEdit.text().split())
        T = np.arange(T_initial, T_final + Tstep, Tstep)
        T = np.r_[T, [298.15]]
        T.sort()

        return T

    def get_P(self):
        Ps = self.ui.lineEdit_2.text().split()
        if len(Ps) == 1:
            return np.array([float(sti) for sti in Ps])
        T_initial, T_final, Tstep = (float(sti) for sti in self.ui.lineEdit_2.text().split())
        T = np.arange(T_initial, T_final + Tstep, Tstep)
        T.sort()

        return T

    def get_FS_T(self):
        return float(self.ui.lineEdit_3.text()), float(self.ui.lineEdit_4.text())

    def set_canvas_configuration(self):

        self.canvas.figure.set_constrained_layout(True)
        self.fig.set_canvas(self.canvas)
        self._ax = self.canvas.figure.add_subplot(111)
        self.cursor = Cursor(self._ax, useblit=True, linewidth=0.8, color='lightgray')

        # Create a scatter plot for the hover marker (initially invisible)
        hover_marker, = self._ax.plot([], [], 'o', color='red', markersize=10, visible=False)

        # Function to check if the cursor is near a datapoint with normalized data
        def is_cursor_near_datapoint(event, line, threshold=0.02):
            """Check if the cursor is near any datapoint for a given line"""
            x_data = normalize(line.get_xdata())  # Retrieve and normalize x data
            y_data = normalize(line.get_ydata())  # Retrieve and normalize y data

            # Normalize the event data as well
            x_cursor = (event.xdata - np.min(line.get_xdata())) / (np.max(line.get_xdata()) - np.min(line.get_xdata()))
            y_cursor = (event.ydata - np.min(line.get_ydata())) / (np.max(line.get_ydata()) - np.min(line.get_ydata()))

            for (xi, yi) in zip(x_data, y_data):
                if (event.xdata is not None) and (event.ydata is not None):
                    distance = np.sqrt((x_cursor - xi) ** 2 + (y_cursor - yi) ** 2)
                    if distance < threshold:
                        # Rescale xi, yi back to original scale for display
                        original_x = xi * (np.max(line.get_xdata()) - np.min(line.get_xdata())) + np.min(
                            line.get_xdata())
                        original_y = yi * (np.max(line.get_ydata()) - np.min(line.get_ydata())) + np.min(
                            line.get_ydata())
                        return original_x, original_y, line.get_label()
            return None, None, None

        # Function to handle mouse movement
        def on_move(event):
            if event.inaxes == self._ax:  # Check if the mouse is within the axes
                # Loop through all lines and check if the cursor is near any datapoint
                for line in self.lines:
                    near_x, near_y, line_label = is_cursor_near_datapoint(event, line)
                    if near_x is not None and near_y is not None:
                        # Update plot title with the coordinates of the datapoint, line label, and x-axis label
                        line_label = line.get_label()
                        str_2_cursor = f'{self._ax.get_xlabel()}: {event.xdata:.2f}, {self._ax.get_ylabel()}: {event.ydata:.2f}\n'
                        str_2_cursor += f'Compound: {self.molecule.formula}\n'
                        str_2_cursor += f'EOS: {self.molecule.eos_str}\nApprox.: {self.modestr}\n'
                        str_2_cursor += f'P:{line_label}\n'
                        self._ax.set_title(str_2_cursor, y=0.95, x=0.01, fontsize=6, loc='left', color='black',
                                           alpha=0.9, va='top')

                        # Update hover marker position and make it visible
                        hover_marker.set_data(near_x, near_y)
                        hover_marker.set_visible(True)
                        self.fig.canvas.draw_idle()  # Redraw the figure to update the title and point colors

                        return

                # If no point is near the cursor, reset the title
                line_label = line.get_label()
                str_2_cursor = f'{self._ax.get_xlabel()}: {event.xdata:.2f}, {self._ax.get_ylabel()}: {event.ydata:.2f}\n'
                # str_2_cursor += f'Compound: {self.molecule.formula}\n'
                # str_2_cursor += f'\nEOS: {self.molecule.eos_str}\nApprox.: {self.modestr}'
                self._ax.set_title(str_2_cursor, y=0.95, x=0.01, fontsize=6, loc='left', color='black', alpha=0.6,
                                   va='top')
                # Reset the color of the markers to the default color when not hovering

                # Hide hover marker when not hovering over any point
                hover_marker.set_visible(False)

                self.fig.canvas.draw_idle()  # Redraw the figure to update the title and point colors

        # Connect the event handler
        self.fig.canvas.mpl_connect('motion_notify_event', on_move)

    def plot_prop(self, str_x, str_y):
        import matplotlib.cm as cm
        self._ax.cla()

        self.data_dict = {pi_str: {} for pi_str in self.dict_tp.keys()}

        len_Ps = len(self.dict_tp.keys())

        self.lines = []
        for i, Pi_str in enumerate(self.dict_tp.keys()):
            tprops_dict = self.dict_tp[Pi_str]

            X = tprops_dict[str_x]
            Y = tprops_dict[str_y]

            self.data_dict[Pi_str] = {str_x: X, str_y: Y}

            c = cm.PuRd((i + 1) / len_Ps, 1)
            line, = self._ax.plot(X, Y, label=Pi_str + 'GPa', color=c)
            self.lines.append(line)
            self._ax.text(X[-1] - 100, Y[-1], 'P=' + Pi_str + 'GPa', size=8)

        self._ax.set_xlabel(str_x)
        self._ax.set_ylabel(str_y)

        self.canvas.draw()

    def check_type_in_energies(self, ti):
        number_of_occurences = 0
        last_occurence = ''
        for key_ai in atom_energy.keys():
            if ti in key_ai:
                number_of_occurences += 1
                last_occurence = key_ai
                # print(key_ai)
        if number_of_occurences == 1:
            return last_occurence
        else:
            return ti

    def debye_run(self, molecule, ui_progress, formula):
        self.formula = formula
        txt4output = f'{formula}$'
        if self.ui.radioButton.isChecked():
            mode = 'jjsl'
            self.modestr = 'Slater'
        elif self.ui.radioButton_2.isChecked():
            mode = 'jjdm'
            self.modestr = 'Dugdale-MacDonald'
        elif self.ui.radioButton_3.isChecked():
            mode = 'jjfv'
            self.modestr = 'FVT'

        self.ui.comboBox.clear()
        self.molecule = molecule
        molecule.initialize_ndeb(mode)
        self.ndeb_obj = molecule.ndeb

        T = self.get_T()
        Ps = self.get_P() * 1e9
        self.Ps = Ps

        self.dict_tp = {'%.1f' % (Pi / 1e9): '' for Pi in Ps}
        self.dict_FS = {'%.1f' % (Pi / 1e9): {'Cp': [0]} for Pi in Ps}
        self.dict_H298 = {'%.1f' % (Pi / 1e9): 0 for Pi in Ps}
        self.dict_S298 = {'%.1f' % (Pi / 1e9): -1 for Pi in Ps}

        lP = len(Ps)
        progress = (0) / lP
        ui_progress.setValue(progress * 100)
        for ix, P in enumerate(Ps):

            self.ui.comboBox.addItem('%.1f' % (P / 1e9))

            molecule.min_G(T, P)

            molecule.eval_props()

            progress = (ix + 1) / lP
            ui_progress.setValue(progress * 100)

            self.dict_tp['%.1f' % (P / 1e9)] = molecule.tprops_dict

            self.FS_Tfrom, self.FS_Tto = self.get_FS_T()

            self.dict_FS['%.1f' % (P / 1e9)] = dt_fit_FS(molecule.tprops_dict, self.FS_Tfrom, self.FS_Tto)

            ix_T0 = np.where(np.round(molecule.tprops_dict['T'], 2) == np.round(298.15, 2))[0][0]

            self.dict_H298['%.1f' % (P / 1e9)] = molecule.tprops_dict['G'][ix_T0] + molecule.tprops_dict['T'][ix_T0] * \
                                                 molecule.tprops_dict['S'][ix_T0]
            self.dict_S298['%.1f' % (P / 1e9)] = molecule.tprops_dict['S'][ix_T0]

            #            self.plot_Cp(self.dict_tp['%.1f'%(P/1e9)], self.dict_FS['%.1f'%(P/1e9)])
            nats = len(molecule.types)
            if P == 0:
                # print(molecule.__dict__.keys())
                Ef = molecule.eos.E0(molecule.eos.V0) - sum([atom_energy[self.check_type_in_energies(ti)] for ti in molecule.types]) * (
                            0.160218e-18 * 6.02214e23) / len(molecule.types)
                self.Ef = Ef*nats
                txt4output += f'{Ef * nats:.7e}'
        self.S298 = self.dict_S298['%.1f' % (0 / 1e9)] * nats
        txt4output += f'${self.dict_S298['%.1f' % (0 / 1e9)] * nats:.7e}$'

        self.ui.tableWidget.setItem(0, 0, QTableWidgetItem('%.5e' % (self.dict_H298['%.1f' % (0 / 1e9)])))
        self.ui.tableWidget.setItem(1, 0, QTableWidgetItem('%.5e' % (self.dict_S298['%.1f' % (0 / 1e9)])))

        lst4output = []
        for ix, p in enumerate(self.dict_FS['%.1f' % (0 / 1e9)]['Cp']):
            lst4output.append(f'{p * nats:.7e}')
            self.ui.tableWidget.setItem(ix + 2, 0, QTableWidgetItem('%.5e' % (p)))
        txt4output += '&'.join(lst4output)
        txt4output += '$'
        t4out = [f'{p:.2e}' for p in [self.FS_Tfrom, self.FS_Tto]]
        txt4output += '&'.join(t4out)

        with open('dtoutput4cmpnd', 'w') as f:
            f.write(txt4output)
        #            self._ax.text(Y[-1],X[-1],'P=')

        #        print('xxxx', list(self.dict_tp[list(self.dict_tp.keys())[0]].keys()))
        self.proplist = list(self.dict_tp[list(self.dict_tp.keys())[0]].keys())
        current_plot = self.current_plot
        self.ui.comboBox_2.clear()

        for k in self.proplist:
            self.ui.comboBox_2.addItem(k)
        self.current_plot = current_plot

        self.ui.comboBox_2.setCurrentIndex(self.current_plot)

        self.plot_prop('T', self.proplist[self.current_plot])

    #        handles_1, labels_1 = self._ax.get_legend_handles_labels()
    #        by_label_1 = dict(zip(labels_1, handles_1))
    #        self._ax.legend(by_label_1.values(), by_label_1.keys())

    def right_menu(self, pos):
        menu = QMenu()

        # Add menu options
        hello_option = menu.addAction('Copy data')

        # Menu option events
        hello_option.triggered.connect(self.on_click_copydata)

        # Position
        menu.exec(self.mapToGlobal(pos))

    def on_pushBack(self):
        self.close()

    def on_click_copydata(self):
        #        app = QApplication(sys.argv)

        txt2copy = ''
        for k in self.data_dict:
            txt2copy = txt2copy + '#P = ' + k + ' GPa\n'
            txt2copy = txt2copy + '#'
            for ki in self.data_dict[k].keys():
                txt2copy = txt2copy + ki + '    '
            txt2copy = txt2copy + '\n'
            data = []
            for ki in self.data_dict[k].keys():
                data.append(self.data_dict[k][ki])

            data = np.array(data).T
            for d in data:
                for di in d:
                    txt2copy = txt2copy + str(di) + '    '
                txt2copy = txt2copy + '\n'
        clipboard = self.app.clipboard()
        clipboard.setText(txt2copy)

    def on_click_recalc(self):
        self._ax.cla()
        self.debye_run(self.molecule, self.ui.progress, self.formula)

    def update_FS_Tto(self):
        try:
            Tf = self.get_T()[-1]
            self.ui.lineEdit_4.setText(str(Tf))
        except:
            pass

    def is_dark_mode(self):
        # Detect if the application is in dark mode using the palette
        palette = self.palette()
        return palette.color(QPalette.ColorRole.Window).value() < 128  # Lightness threshold for dark mode

    def on_text_changed(self, line_edit):
        # Call the reusable highlight function
        dark_mode = self.is_dark_mode()
        color_p = "purple" if dark_mode else "#94a2c9"

        highlight_line_edit(line_edit, color=color_p, duration=500)
        # Update label with the current content of the edited line edit



import numpy as np
from scipy.optimize import leastsq

class Xs:
    def __init__(self, *params: np.ndarray):
        self.xs0, self.xs1, self.xs2, self.xs3, self.xs4, self.xs5 = params

    def E(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Electronic energy

        :param T: Temperature.
        :type T: float|np.ndarray
        :param V: Volume.
        :type V: float|np.ndarray
        :return: E_el
        :rtype: float|np.ndarray
        """
        return self.F(T, V) + T*self.S(T, V)

    def S(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Electronic entropy.

        :param T: Temperature.
        :type T: float|np.ndarray
        :param V: Volume.
        :type V: float|np.ndarray
        :return: S_el
        :rtype: float|np.ndarray
        """
        return -(self.xs1 + 2 * self.xs2 * T + 3 * self.xs3 * T**2 + self.xs4 * (np.log(T) + 1) - 2 * self.xs5 * T**(-3))

    def F(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return self.xs0 + self.xs1 * T + self.xs2 * T**2 + self.xs3 * T**3 + self.xs4 * T * np.log(T) + self.xs5 * T**(-2)


    def dFdV_T(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return 0
    def dFdT_V(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return self.xs1 + 2 * self.xs2 * T + 3 * self.xs3 * T**2 + self.xs4 * (np.log(T) + 1) - 2 * self.xs5 * T**(-3)

    def d2FdT2_V(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return 2 * self.xs2 + 6 * self.xs3 * T + self.xs4 / T + 6 * self.xs5 * T**(-4)

    def d2FdV2_T(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return 0
    def d3FdV3_T(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return 0
    def d4FdV4_T(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return 0

    def d2FdVdT(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return 0
    def d3FdV2dT(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return 0
    def d3FdVdT2(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return 0





from PySide6.QtWidgets import QLineEdit, QPushButton
from PySide6.QtCore import Signal, Qt
from PySide6.QtGui import QMouseEvent


class ClickableLineEdit(QLineEdit):
    # Define a custom signal named 'clicked'
    clicked = Signal()

    def __init__(self, parent=None):
        super(ClickableLineEdit, self).__init__(parent)

    def mousePressEvent(self, event):
        self.clicked.emit()  # Emit the clicked signal on mouse press
        super(ClickableLineEdit, self).mousePressEvent(event)

element_colors = {
    "H": "#00FF00", "He": "#EE82EE",
    "Li": "#FF6666", "Be": "#FFA500", "B": "#FFFF99", "C": "#00FF00", "N": "#00FF00", "O": "#00FF00", "F": "#FFFF00", "Ne": "#EE82EE",
    "Na": "#FF6666", "Mg": "#FFA500", "Al": "#FFDAB9", "Si": "#FFFF99", "P": "#00FF00", "S": "#00FF00", "Cl": "#FFFF00", "Ar": "#EE82EE",
    "K": "#FF6666", "Ca": "#FFA500", "Sc": "#ADD8E6", "Ti": "#ADD8E6", "V": "#ADD8E6", "Cr": "#ADD8E6", "Mn": "#ADD8E6", "Fe": "#ADD8E6",
    "Co": "#ADD8E6", "Ni": "#ADD8E6", "Cu": "#ADD8E6", "Zn": "#ADD8E6", "Ga": "#FFDAB9", "Ge": "#FFFF99", "As": "#FFFF99", "Se": "#00FF00",
    "Br": "#FFFF00", "Kr": "#EE82EE", "Rb": "#FF6666", "Sr": "#FFA500", "Y": "#ADD8E6", "Zr": "#ADD8E6", "Nb": "#ADD8E6", "Mo": "#ADD8E6",
    "Tc": "#ADD8E6", "Ru": "#ADD8E6", "Rh": "#ADD8E6", "Pd": "#ADD8E6", "Ag": "#ADD8E6", "Cd": "#ADD8E6", "In": "#FFDAB9", "Sn": "#FFDAB9",
    "Sb": "#FFFF99", "Te": "#FFFF99", "I": "#FFFF00", "Xe": "#EE82EE", "Cs": "#FF6666", "Ba": "#FFA500", "La": "#FFB6C1", "Ce": "#FFB6C1",
    "Pr": "#FFB6C1", "Nd": "#FFB6C1", "Pm": "#FFB6C1", "Sm": "#FFB6C1", "Eu": "#FFB6C1", "Gd": "#FFB6C1", "Tb": "#FFB6C1", "Dy": "#FFB6C1",
    "Ho": "#FFB6C1", "Er": "#FFB6C1", "Tm": "#FFB6C1", "Yb": "#FFB6C1", "Lu": "#FFB6C1", "Hf": "#ADD8E6", "Ta": "#ADD8E6", "W": "#ADD8E6",
    "Re": "#ADD8E6", "Os": "#ADD8E6", "Ir": "#ADD8E6", "Pt": "#ADD8E6", "Au": "#ADD8E6", "Hg": "#ADD8E6", "Tl": "#FFDAB9", "Pb": "#FFDAB9",
    "Bi": "#FFDAB9", "Po": "#FFDAB9", "At": "#FFFF00", "Rn": "#EE82EE", "Fr": "#FF6666", "Ra": "#FFA500", "Ac": "#D8BFD8", "Th": "#D8BFD8",
    "Pa": "#D8BFD8", "U": "#D8BFD8", "Np": "#D8BFD8", "Pu": "#D8BFD8", "Am": "#D8BFD8", "Cm": "#D8BFD8", "Bk": "#D8BFD8", "Cf": "#D8BFD8",
    "Es": "#D8BFD8", "Fm": "#D8BFD8", "Md": "#D8BFD8", "No": "#D8BFD8", "Lr": "#D8BFD8", "Rf": "#ADD8E6", "Db": "#ADD8E6", "Sg": "#ADD8E6",
    "Bh": "#ADD8E6", "Hs": "#ADD8E6", "Mt": "#ADD8E6", "Ds": "#ADD8E6", "Rg": "#ADD8E6", "Cn": "#ADD8E6", "Nh": "#FFDAB9", "Fl": "#FFDAB9",
    "Mc": "#FFDAB9", "Lv": "#FFDAB9", "Ts": "#FFFF00", "Og": "#EE82EE"
}

class CountingButton(QPushButton):
    counterChanged = Signal(int)  # Signal that indicates the counter has changed
    counterChanged2 = Signal(str, int)

    def __init__(self, label, parent=None):
        super(CountingButton, self).__init__(label, parent)
        self.counter = 0

    def set_style(self):
        bc = 'None' if self.counter==0 else 'purple'
        bw = 1 if self.counter==0 else 2
        # print(bc)
        self.setStyleSheet(
                'QPushButton {' + \
                f'background-color: {element_colors[self.text()]};' + \
                    'border-style: outset;' + \
                    f'border-width: {bw}px;' + \
                    'border-radius: 5px;' + \
                    'color: gray;' + \
                    # 'border-color: beige;' + \
                    'padding: 0px;' + \
                    f'border-color: {bc};' + \
                '}' + \
                'QPushButton:pressed {' + \
                    'background-color: navy;' + \
                    'border-style: inset;' + \
                    'border-color: purple;' + \
                '}' + \
                ''
            )

    def mousePressEvent(self, event: QMouseEvent):
        if event.button() == Qt.LeftButton:
            self.incrementCounter()
        elif event.button() == Qt.RightButton:
            self.decrementCounter()
        super(CountingButton, self).mousePressEvent(event)

    def incrementCounter(self):
        self.counter += 1
        self.counter = max(self.counter, 0)
        self.counterChanged2.emit(self.text(), self.counter) #WIP Emit Text+counter
        # self.updateText()

    def decrementCounter(self):
        self.counter -= 1
        self.counter = max(self.counter, 0)
        self.counterChanged2.emit(self.text(), self.counter)
        # self.updateText()

    # def updateText(self):
    #     self.setText(f"Count: {self.counter}")



from PySide6.QtWidgets import QDialog, QFileDialog
from debyetools.tpropsgui.ui_dialog_loadEOS import Ui_Dialog as Ui_loadEOS
# from atomtools import atomic_mass
from debyetools.aux_functions import load_V_E as dt_load_V_E

from PySide6.QtCore import QTimer
from PySide6.QtGui import QPixmap, QPalette
# def log_mean(lst):
#     import numpy as np
#     l = len(lst)
#     # lm = np.sum([np.log(li)/l for li in lst])
#     # m = np.exp(lm)
#     return np.sum([li for li in lst])/l


def highlight_line_edit(line_edit, color="purple", duration=100):
    # Set the background color
    line_edit.setStyleSheet(f"background-color: {color};")

    # Create a QTimer to reset the color after `duration` milliseconds
    timer = QTimer(line_edit)
    timer.setSingleShot(True)  # Only trigger once
    timer.timeout.connect(lambda: line_edit.setStyleSheet(""))  # Reset the color
    timer.start(duration)

class dialogLoadEOS(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_loadEOS()
        self.ui.setupUi(self)

        self.ui.browsesummary.clicked.connect(self.getfilessummary)
        self.ui.browseposcar.clicked.connect(self.getfilesposcar)
        self.ui.ok.clicked.connect(self.on_pushButton_OK)
        self.filepath='.'

        # Connect the textChanged signals to a shared color change method
        self.ui.summarypath.textChanged.connect(lambda: self.on_text_changed(self.ui.summarypath))
        self.ui.poscarpath.textChanged.connect(lambda: self.on_text_changed(self.ui.poscarpath))


    def getfilessummary(self):
        summarypath, _ = QFileDialog.getOpenFileName(self, caption='Select a SUMMARY file')
        self.ui.summarypath.setText(summarypath)
        self.summarypath = summarypath

    def getfilesposcar(self):
        poscarpath, _ = QFileDialog.getOpenFileName(self, caption='Select a POSCAR or CONTCAR file')
        self.ui.poscarpath.setText(poscarpath)
        self.poscarpath = poscarpath

    def on_pushButton_OK(self):
        self.summarypath = self.ui.summarypath.text()
        self.poscarpath = self.ui.poscarpath.text()
        self.close()

    def closeEvent(self, event):
        V, E = dt_load_V_E(self.summarypath, self.poscarpath)
        with open(self.poscarpath) as f:
            poscar_lines = f.readlines()
        elmnts = poscar_lines[5].split()
        mult = poscar_lines[6].split()
        elmntsall = []
        for e, m in zip(elmnts, mult):
            elmntsall = elmntsall + [e]*int(m)
        # masses = [float(atomic_mass[e])/1000. for e in elmntsall]
#        print(elmntsall, masses, log_mean(masses))
        txt2paste = '#V\tE\n'
        for v, e in zip(V, E):
            txt2paste = txt2paste + '%.6e\t%.6e'%(v, e)+'\n'
        self.EvVtext.setPlainText(txt2paste)
        # self.mass.setText(str(log_mean(masses)))
#        print('event', event)

    def is_dark_mode(self):
        # Detect if the application is in dark mode using the palette
        palette = self.palette()
        return palette.color(QPalette.ColorRole.Window).value() < 128  # Lightness threshold for dark mode

    def on_text_changed(self, line_edit):
        # Call the reusable highlight function
        dark_mode = self.is_dark_mode()
        color_p = "purple" if dark_mode else "#94a2c9"

        highlight_line_edit(line_edit, color=color_p, duration=500)




from debyetools.tpropsgui.atomtools import atomic_mass
import pandas as pd
from debyetools.aux_functions import load_doscar
from debyetools.get_elastic import get_EM


class Vdata:
    def __init__(self):
        pass


def parse_contcar(file_path):
    try:
        with open(file_path, 'r') as f:
            lines = [line.strip() for line in f if line.strip()]
    except FileNotFoundError:
        print(f"Error: File '{file_path}' not found.")

    if len(lines) < 7:
        print("Error: The CONTCAR file is too short to be valid.")

    line6 = lines[5]
    line7 = lines[6]

    # Determine if line6 contains species or counts
    if all(item.isdigit() for item in line6.split()):
        # Line6 contains counts, species not provided
        print("Error: Element symbols not provided in the CONTCAR file.")
    else:
        species = line6.split()
        counts = list(map(int, line7.split()))
        if len(species) != len(counts):
            print("Error: The number of species and counts do not match.")

        formula = []
        for elem, count in zip(species, counts):
            formula.extend([elem] * count)

        return formula


def average_mass(elements):
    # Calculate the sum of the masses of the elements in the list
    atomic_mass['VA'] = 0
    total_mass = sum(atomic_mass[element] for element in elements)
    # Calculate the average mass
    average = total_mass / len(elements)
    return average


def load_energies(file_path):
    # Read the data into a DataFrame, skipping the first line and using whitespace as the delimiter
    df = pd.read_csv(file_path, skiprows=1, sep=r'\s+', header=None)
    # Assign column names based on the header information in the data
    df.columns = ["Element", "Structure", "Total-energy", "Mag", "A-conv", "Vol-conv", "Vol-at", "R-at", "B/A", "C/A"]

    return df


def get_energy(potential, current_path):
    energies_df = load_energies(f'{current_path}/elements_energies.out')
    # Query the DataFrame to find the total energy of the element
    total_energy = energies_df[energies_df['Element'] == potential]['Total-energy'].iloc[0]
    # print(f"Energy of {element}: {total_energy}")
    return total_energy


def extract_from_DFT(file_path):
    vdata = Vdata()

    # Extract total energy from DFT calculations
    path = file_path  # '.'
    E = []
    V = []
    nats = 0
    E0 = 0
    vi = 70
    vf = 130
    step = 3
    potentials_set = []
    for i in range(vi, vf + step, step):
        try:
            with open(f'{path}/EvV/{i}/OUTCAR') as f:
                Ei = 0
                Vi = 0
                natsi = 0
                lines = f.readlines()
            for line in lines:
                if 'volume of cell' in line:
                    Vi = float(line.split()[-1])
                if 'TOTEN' in line:
                    Ei = float(line.split()[4])
                if 'NIONS' in line:
                    natsi = float(line.split()[-1])
                if 'POTCAR:' in line:
                    potentials_set.append(line.split()[2])
            E.append(Ei / natsi)
            V.append(Vi / natsi)
            nats = natsi
            if i == 100:
                E0 = Ei / natsi

        except Exception as e:
            print(f'Warning [process_configurations]: {e}\n')
    vdata.V = V
    vdata.E = E
    potentials_set = set(potentials_set)
    vdata.potentials = {p.split('_')[0]: p for p in potentials_set}
    vdata.potentials['VA'] = 'VA'

    vdata.formula = parse_contcar(f'{path}/relaxation/CONTCAR')
    vdata.nats = nats
    # compound.multiplicities = multiplicities
    vdata.mass = average_mass(vdata.formula) / 1000
    current_folder = f'{path}'
    vdata.Ef = E0 - sum([get_energy(vdata.potentials[fi], current_folder) for fi in vdata.formula]) / nats  # sum(multiplicities)
    vdata.E0 = E0
    vdata.path = path

    list_filetags = [f'/EvV/{i}/DOSCAR' for i in range(70, 130 + 3, 3)]
    vdata.electric = load_doscar(path, list_filetags=list_filetags)

    EM = get_EM(f'{path}/elastic')
    # try:
    #     EM = load_EM(f'{path}/elastic/OUTCAR')
    # except Exception as e:
    #     EM = None
    #     print( f'Warning [process_configurations]: Elastic Moduli not found for {path}.\n')
    #     print(e)
    vdata.EM = EM

    return vdata


from matplotlib import pyplot as plt
from debyetools.tpropsgui.elastic_props import run_script_plots
from debyetools.aux_functions import load_EM


EM = load_EM('../Al3Li_L12/OUTCAR_elastic')/10

fig, ax = plt.subplots(4,3, subplot_kw={'projection': 'polar'}, figsize=(8,12), layout='constrained')
ax = run_script_plots(ax, EM)
plt.show()


from PySide6.QtWidgets import QDialog

from debyetools.tpropsgui.ui_dialog_plots_elastic import Ui_Dialog as Ui_plots

from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5agg import FigureCanvas
import numpy as np


def print_to_box(ptedit, txt=''):
    txt = ptedit.toPlainText()+txt
    ptedit.setPlainText(txt)
    ptedit.verticalScrollBar().setValue(ptedit.verticalScrollBar().maximum())

class dialogPlots(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_plots()
        self.ui.setupUi(self)

        self.fig1 = Figure(figsize=(9, 3))
        self.fig2 = Figure(figsize=(9, 3))
        self.fig3 = Figure(figsize=(9, 3))
        self.fig4 = Figure(figsize=(9, 3))

        self.canvas1 = FigureCanvas(self.fig1)
        self.canvas2 = FigureCanvas(self.fig2)
        self.canvas3 = FigureCanvas(self.fig3)
        self.canvas4 = FigureCanvas(self.fig4)

        self.canvas_configuration()

        llayout1 = self.ui.hLplots1
        llayout1.addWidget(self.canvas1, 88)

        llayout2 = self.ui.hLplots2
        llayout2.addWidget(self.canvas2, 100)

        llayout3 = self.ui.hLplots3
        llayout3.addWidget(self.canvas3, 80)

        llayout4 = self.ui.hLplots4
        llayout4.addWidget(self.canvas4, 100)

    def canvas_configuration(self):
        self.fig1.set_canvas(self.canvas1)
        self.fig2.set_canvas(self.canvas2)
        self.fig3.set_canvas(self.canvas3)
        self.fig4.set_canvas(self.canvas4)

        self.ax1 = []
        self.ax2 = []
        self.ax3 = []
        self.ax4 = []
        for i in range(3):
            self.ax1.append(self.canvas1.figure.add_subplot(1, 3, i+1, polar=True))
            self.ax2.append(self.canvas2.figure.add_subplot(1, 3, i+1, polar=True))
            self.ax3.append(self.canvas3.figure.add_subplot(1, 3, i+1, polar=True))
            self.ax4.append(self.canvas4.figure.add_subplot(1, 3, i+1, polar=True))

            # axi.set_axis_off()

        # self.fig.subplots_adjust(hspace=0.05, right=0.99, top=.99, bottom=.01, left=0.01, wspace=0.05)




from PySide6.QtWidgets import QDialog, QMessageBox

from debyetools.tpropsgui.ui_dialog_calculateNu import Ui_Form as Ui_iparams

from debyetools.poisson import poisson_ratio as dt_poisson_ratio

import numpy as np

from debyetools.tpropsgui.dialog_loadElastic import dialogLoadElastic

from PySide6.QtCore import QTimer
from PySide6.QtGui import QPixmap, QPalette

from debyetools.tpropsgui.dialog_elastic import dialogElastic

import debyetools.tpropsgui.elastic_props as elastic
import sys
def highlight_line_edit(line_edit, color="purple", duration=100):
    # Set the background color
    line_edit.setStyleSheet(f"background-color: {color};")

    # Create a QTimer to reset the color after `duration` milliseconds
    timer = QTimer(line_edit)
    timer.setSingleShot(True)  # Only trigger once
    timer.timeout.connect(lambda: line_edit.setStyleSheet(""))  # Reset the color
    timer.start(duration)


class dialogCalcNu(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_iparams()
        self.ui.setupUi(self)

        self.ui.pushSave.clicked.connect(self.on_pushSave)
        self.ui.pushButton_2.clicked.connect(self.on_pushButton_calc_nu)

        self.ui.pushLoadElastic.clicked.connect(self.on_pushLoadElastic)

        self.dialog_loadElastic = dialogLoadElastic(self)

        # Connect the textChanged signals to a shared color change method
        self.ui.elastic_constants.textChanged.connect(lambda: self.on_text_changed(self.ui.elastic_constants))
        self.ui.lineEdit_5.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_5))
        self.ui.lineEdit_4.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_4))
        self.ui.lineEdit_6.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_6))
        self.ui.lineEdit_10.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_10))
        self.ui.lineEdit_9.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_9))
        self.ui.lineEdit_8.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_8))
        self.ui.lineEdit_7.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_7))
        self.ui.lineEdit_3.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_3))

        self.ui.pushMore.clicked.connect(self.on_pushMore_calcElastic)

        self.dialogElastic = dialogElastic(self)

    def on_pushLoadElastic(self):
        self.dialog_loadElastic.elastic_constants = self.ui.elastic_constants
        self.dialog_loadElastic.show()

    def on_pushSave(self):
        current_text = self.ui.lineEdit_3.text()
        self.external_nu.setText(current_text)
        self.close()

    def get_C(self):
        txt = self.ui.elastic_constants.toPlainText().replace('XX', ' ').replace('YY', ' ').replace('ZZ', ' ').replace(
            'XY', ' ').replace('YZ', ' ').replace('ZX', ' ').split('\n')
        data_lst = []
        for ti in txt:
            if len(ti) == 0: continue
            if ti[0] == '#': continue
            data_lst.append([float(tii) for tii in ti.replace('\t', ' ').split()])

        # print(data_lst)
        data = np.array(data_lst)

        return data

    def on_pushButton_calc_nu(self):
        C = self.get_C()
        BR, BV, B, GR, GV, S, AU, nu = dt_poisson_ratio(C, quiet=True)
        self.ui.lineEdit_3.setText('%.3f' % (nu))
        self.ui.lineEdit_4.setText('%.1f' % (BR * 10))
        self.ui.lineEdit_5.setText('%.1f' % (BV * 10))
        self.ui.lineEdit_6.setText('%.1f' % (B * 10))
        self.ui.lineEdit_9.setText('%.1f' % (GV * 10))
        self.ui.lineEdit_8.setText('%.1f' % (GR * 10))
        self.ui.lineEdit_7.setText('%.1f' % (S * 10))
        self.ui.lineEdit_10.setText('%.2f' % (AU))

    def is_dark_mode(self):
        # Detect if the application is in dark mode using the palette
        palette = self.palette()
        return palette.color(QPalette.ColorRole.Window).value() < 128  # Lightness threshold for dark mode

    def on_text_changed(self, line_edit):
        # Call the reusable highlight function
        dark_mode = self.is_dark_mode()
        color_p = "purple" if dark_mode else "#94a2c9"

        highlight_line_edit(line_edit, color=color_p, duration=500)
        # Update label with the current content of the edited line edit

    def on_pushMore_calcElastic(self):

        sys.stdout.reconfigure(encoding='utf-8')

        C = self.get_C()
        res, resdata=elastic.run_script(C)
        del resdata

        print(res.expandtabs(8))

        # # Assuming a font width of 8 pixels per character for 8 spaces
        # tab_width = 8 * 8  # 8 spaces * approx. 8 pixels per space in typical monospace font

        # # Set the tab width in the QPlainTextEdit
        # self.dialogElastic.ui.plainTextEdit.setTabStopDistance(tab_width)

        # Set the text content with expandtabs(8) to replace tabs with 8 spaces in the content itself
        self.dialogElastic.ui.plainTextEdit.setPlainText(res.expandtabs(8))
        self.dialogElastic.EM = C
        self.dialogElastic.show()


        # QMessageBox.information(self, 'Error', res.expandtabs(8), QMessageBox.Ok)




from debyetools.aux_functions import load_cell
from debyetools.pairanalysis import pair_analysis
formula, primitive_cell, basis_vectors = load_cell('../Al3Li_L12/CONTCAR')
cutoff = 5
pa_result = pair_analysis(formula, cutoff, basis_vectors, primitive_cell)
distances , num_pairs_per_formula , combs_types = pa_result
print('distances| # of pairs per type')
print('         | '+'  '.join(['%s' for _ in combs_types])%tuple(combs_types))
for d, n in zip( distances , num_pairs_per_formula ):
    print(' %.6f' % (d) + '| ' + ' '.join([' %.2f' for _ in n])%tuple(n))


from PySide6.QtWidgets import  QDialog, QFileDialog
from debyetools.tpropsgui.ui_dialog_loadElastic import Ui_Dialog as Ui_OUTCAR
from debyetools.aux_functions import load_EM as dt_load_EM

from PySide6.QtCore import QTimer
from PySide6.QtGui import QPixmap, QPalette

def highlight_line_edit(line_edit, color="purple", duration=100):
    # Set the background color
    line_edit.setStyleSheet(f"background-color: {color};")

    # Create a QTimer to reset the color after `duration` milliseconds
    timer = QTimer(line_edit)
    timer.setSingleShot(True)  # Only trigger once
    timer.timeout.connect(lambda: line_edit.setStyleSheet(""))  # Reset the color
    timer.start(duration)

class dialogLoadElastic(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_OUTCAR()
        self.ui.setupUi(self)

        self.ui.browseoutcar.clicked.connect(self.getfilesoutcar)
#        self.ui.browseposcar.clicked.connect(self.getfilesposcar)
        self.ui.ok.clicked.connect(self.on_pushButton_OK)
        self.filepath='.'

        # Connect the textChanged signals to a shared color change method
        self.ui.outcarpath.textChanged.connect(lambda: self.on_text_changed(self.ui.outcarpath))


    def getfilesoutcar(self):
        outcarpath, _ = QFileDialog.getOpenFileName(self, caption='Select an OUTCAR file')
        self.ui.outcarpath.setText(outcarpath)

#    def getfilesposcar(self):
#        poscarpath, _ = QFileDialog.getOpenFileName(self, caption='Select a POSCAR or CONTCAR file')
#        self.ui.poscarpath.setText(poscarpath)

    def on_pushButton_OK(self):
        self.outcarpath = self.ui.outcarpath.text()
#        self.poscarpath = self.ui.poscarpath.text()
        self.close()
    def closeEvent(self, event):
        EM = dt_load_EM(self.outcarpath)
#        print(EM)

        txt2paste = ''
        for rowi in EM:
            txt2paste=txt2paste+' '.join(['%.2f'%(float(coli)/10) for coli in rowi])+'\n'
#        print('event', event)
        self.elastic_constants.setText(txt2paste)

    def is_dark_mode(self):
        # Detect if the application is in dark mode using the palette
        palette = self.palette()
        return palette.color(QPalette.ColorRole.Window).value() < 128  # Lightness threshold for dark mode

    def on_text_changed(self, line_edit):
        # Call the reusable highlight function
        dark_mode = self.is_dark_mode()
        color_p = "purple" if dark_mode else "#94a2c9"

        highlight_line_edit(line_edit, color=color_p, duration=500)
        # Update label with the current content of the edited line edit



# This Python file uses the following encoding: utf-8

# if __name__ == "__main__":
#     pass



import numpy as np
from scipy import optimize
from scipy.optimize import minimize
from itertools import product
"""
This module contains functions to calculate elastic properties from the elastic constants
as in ELATE by Romain Gaillac and François-Xavier Coudert.
"""

def dirVec(theta, phi):
    """Return a unit vector associated with angles theta and phi"""
    return [np.sin(theta)*np.cos(phi), np.sin(theta)*np.sin(phi), np.cos(theta)]
def dirVec2(theta, phi, chi):
    """Return the second unit vector associated with angles (theta, phi, chi)"""
    return [np.cos(theta)*np.cos(phi)*np.cos(chi) - np.sin(phi)*np.sin(chi),
            np.cos(theta)*np.sin(phi)*np.cos(chi) + np.cos(phi)*np.sin(chi),
            - np.sin(theta)*np.cos(chi)]

def SVoigtCoeff(p,q):
            return 1. / ((1+p//3)*(1+q//3))
def calc_Smat(Cs):
    mat = np.array(Cs)
    # Check that is is symmetric, or make it symmetric
    if np.linalg.norm(np.tril(mat, -1)) == 0:
        mat = mat + np.triu(mat, 1).transpose()
    if np.linalg.norm(np.triu(mat, 1)) == 0:
        mat = mat + np.tril(mat, -1).transpose()
    if np.linalg.norm(mat - mat.transpose()) > 1e-3:
        raise ValueError("should be symmetric, or triangular")
    elif np.linalg.norm(mat - mat.transpose()) > 0:
        # It was almost symmetric: symmetrize it completely
        mat = 0.5 * (mat + mat.transpose())

    VoigtMat = [[0, 5, 4], [5, 1, 3], [4, 3, 2]]
    # CVoigt = mat
    SVoigt = np.linalg.inv(Cs)
    return [[[[ SVoigtCoeff(VoigtMat[i][j], VoigtMat[k][l]) * SVoigt[VoigtMat[i][j]][VoigtMat[k][l]]
                         for i in range(3) ] for j in range(3) ] for k in range(3) ] for l in range(3) ]

def Young_tp(x, y, Smat):
    a = dirVec(x, y)
    r = sum([ a[i]*a[j]*a[k]*a[l] * Smat[i][j][k][l]
                for i in range(3) for j in range(3) for k in range(3) for l in range(3) ])
    return 1/r

def LinearCompressibility_tp(x, y, Smat):
        a = dirVec(x, y)
        r = sum([ a[i]*a[j] * Smat[i][j][k][k]
                  for i in range(3) for j in range(3) for k in range(3) ])
        return 1000 * r

def shear(x, Smat):
        a = dirVec(x[0], x[1])
        b = dirVec2(x[0], x[1], x[2])
        r = sum([ a[i]*b[j]*a[k]*b[l] * Smat[i][j][k][l]
                  for i in range(3) for j in range(3) for k in range(3) for l in range(3) ])
        return 1/(4*r)

def Poisson(x, Smat):
        a = dirVec(x[0], x[1])
        b = dirVec2(x[0], x[1], x[2])
        r1 = sum([ a[i]*a[j]*b[k]*b[l] * Smat[i][j][k][l]
                   for i in range(3) for j in range(3) for k in range(3) for l in range(3) ])
        r2 = sum([ a[i]*a[j]*a[k]*a[l] * Smat[i][j][k][l]
                   for i in range(3) for j in range(3) for k in range(3) for l in range(3) ])
        return -r1/r2

import sys
def f1f2(x, Smat, fname):
     funct = getattr(sys.modules[__name__], fname)
     return lambda z: funct([x[0], x[1], z[0]], Smat), lambda z: -funct([x[0], x[1], z[0]], Smat)

def shear2D(x, Smat):
    ftol = 0.001
    xtol = 0.01
    func1, func2 = f1f2([x[0], x[1]], Smat, 'shear')

    r1 = optimize.minimize(func1, np.pi/2.0, method = 'Powell', options={"xtol":xtol, "ftol":ftol})
    r2 = optimize.minimize(func2, np.pi/2.0, method = 'Powell', options={"xtol":xtol, "ftol":ftol})
    return float(r1.fun), -float(r2.fun)

def Poisson2D(x, Smat):
    ftol = 0.001
    xtol = 0.01
    func1, func2 = f1f2([x[0], x[1]], Smat, 'Poisson')

    r1 = optimize.minimize(func1, np.pi/2.0, method = 'Powell', options={"xtol":xtol, "ftol":ftol})
    r2 = optimize.minimize(func2, np.pi/2.0, method = 'Powell', options={"xtol":xtol, "ftol":ftol})
    return min(0,float(r1.fun)), max(0,float(r1.fun)), -float(r2.fun)

def averages(CVoigt):
    SVoigt = np.linalg.inv(CVoigt)
    A = (CVoigt[0][0] + CVoigt[1][1] + CVoigt[2][2]) / 3
    B = (CVoigt[1][2] + CVoigt[0][2] + CVoigt[0][1]) / 3
    C = (CVoigt[3][3] + CVoigt[4][4] + CVoigt[5][5]) / 3
    a = (SVoigt[0][0] + SVoigt[1][1] + SVoigt[2][2]) / 3
    b = (SVoigt[1][2] + SVoigt[0][2] + SVoigt[0][1]) / 3
    c = (SVoigt[3][3] + SVoigt[4][4] + SVoigt[5][5]) / 3

    KV = (A + 2*B) / 3
    GV = (A - B + 3*C) / 5

    KR = 1 / (3*a + 6*b)
    GR = 5 / (4*a - 4*b + 3*c)

    KH = (KV + KR) / 2
    GH = (GV + GR) / 2

    return [ [KV, 1/(1/(3*GV) + 1/(9*KV)), GV, (1 - 3*GV/(3*KV+GV))/2],
                [KR, 1/(1/(3*GR) + 1/(9*KR)), GR, (1 - 3*GR/(3*KR+GR))/2],
                [KH, 1/(1/(3*GH) + 1/(9*KH)), GH, (1 - 3*GH/(3*KH+GH))/2] ]

# Define Born stability criteria for cubic crystals
def check_born_stability(Cmat):
    c11, c12, c44 = Cmat[0][0], Cmat[0][1], Cmat[3][3]
    stability = f'c11 > 0: {c11 > 0}\nc44 > 0: {c44 > 0}\nc11 - c12 > 0: {c11 - c12 > 0}\nc11 + 2*c12 > 0: {c11 + 2 * c12 > 0}'
    return stability



def get_min_max_directions(Smat, fstr, opt_ix=0):
    t = np.linspace(0, 2*np.pi, 5)
    p = np.linspace(0, 2*np.pi, 5)
    initial_guesses = list(product(t, p))
    # initial_guesses = [[0,0],[np.pi/2, np.pi], [0, np.pi/3], [0, 2*np.pi/3], [0, 3*np.pi/3], [0, 4*np.pi/3], [0, 5*np.pi/3], [0, 6*np.pi/3],
    #                 [np.pi/3,0], [np.pi/3, np.pi/3], [np.pi/3, 2*np.pi/3], [np.pi/3, 3*np.pi/3], [np.pi/3, 4*np.pi/3], [np.pi/3, 5*np.pi/3], [np.pi/3, 6*np.pi/3],
    #                     [2*np.pi/3,0], [2*np.pi/3, np.pi/3], [2*np.pi/3, 2*np.pi/3], [2*np.pi/3, 3*np.pi/3], [2*np.pi/3, 4*np.pi/3], [2*np.pi/3, 5*np.pi/3], [2*np.pi/3, 6*np.pi/3],
    #                     [3*np.pi/3,0], [3*np.pi/3, np.pi/3], [3*np.pi/3, 2*np.pi/3], [3*np.pi/3, 3*np.pi/3], [3*np.pi/3, 4*np.pi/3], [3*np.pi/3, 5*np.pi/3], [3*np.pi/3, 6*np.pi/3],
    #                     [4*np.pi/3,0], [4*np.pi/3, np.pi/3], [4*np.pi/3, 2*np.pi/3], [4*np.pi/3, 3*np.pi/3], [4*np.pi/3, 4*np.pi/3], [4*np.pi/3, 5*np.pi/3], [4*np.pi/3, 6*np.pi/3],
    #                     [5*np.pi/3,0], [5*np.pi/3, np.pi/3], [5*np.pi/3, 2*np.pi/3], [5*np.pi/3, 3*np.pi/3], [5*np.pi/3, 4*np.pi/3], [5*np.pi/3, 5*np.pi/3], [5*np.pi/3, 6*np.pi/3],
    #                     [6*np.pi/3,0], [6*np.pi/3, np.pi/3], [6*np.pi/3, 2*np.pi/3], [6*np.pi/3, 3*np.pi/3], [6*np.pi/3, 4*np.pi/3], [6*np.pi/3, 5*np.pi/3], [6*np.pi/3, 6*np.pi/3]]
    minf = 1000
    maxf = -1000

    funct = getattr(sys.modules[__name__], fstr)

    # Get the number of arguments
    num_args = funct.__code__.co_argcount

    if num_args == 3:
        Young_funct = lambda t, p: funct(t, p, Smat)
        f2min = lambda x: abs(Young_funct(x[0], x[1]))
        f2max = lambda x: -abs(Young_funct(x[0], x[1]))
    elif num_args == 2:
        Young_funct = lambda t: funct(t, Smat)[opt_ix]
        f2min = lambda x: abs(Young_funct(x))
        f2max = lambda x: -abs(Young_funct(x))
    current_direction_min = [0,0]
    current_direction_max = [0,0]
    for initial_guess in initial_guesses:
        result = minimize(f2min, initial_guess, method='BFGS')
        if result.fun < minf:
            minf = result.fun
            current_direction_min = result.x
        result = minimize(f2max, initial_guess, method='BFGS')
        if -result.fun > maxf:
            maxf = - result.fun
            current_direction_max = result.x

    dir_min = dirVec(current_direction_min[0], current_direction_min[1])
    dir_max = dirVec(current_direction_max[0], current_direction_max[1])
    return minf, maxf, dir_min, dir_max

def run_script(EM):
    # input
    resdata = {}
    txt2output = ''
    txt2output =txt2output + 'Stiffness matrix: (in GPa)\n'
    for c in EM:
        for ci in c:
            txt2output = txt2output + f'{ci:.3f}' + '\t'
        txt2output = txt2output + '\n'
    txt2output = txt2output + '\n'
    resdata['Cijs'] = EM

    # Check stability
    # stability_results = check_born_stability(EM)
    # txt2output = txt2output + 'Born staibily: \n'+ stability_results + '\n'
    # txt2output = txt2output + '\n'
    # resdata['Born_stability'] = stability_results

    #- Eigenvalues of the stiffness matrix (lambda 1 to 6).
    # Calculate the eigenvalues of the stiffness matrix
    eigenvalues = np.linalg.eigvals(EM)
    # Display the eigenvalues
    txt2output = txt2output + "Eigenvalues of the stiffness matrix (in GPa):" + '\n'

    txt2output = txt2output + '\t'.join([f"λ{i+1}" for i in range(len(eigenvalues))])+ '\n'
    txt2output = txt2output + '\t'.join([f"{eigenvalue.real:.2f}" for eigenvalue in eigenvalues])+ '\n'
    if np.all(eigenvalues > 0):
        txt2output = txt2output + 'The material is mechanically stable' + '\n'
    txt2output = txt2output + '\n'

    resdata['eigenvalues'] = eigenvalues


    #Average properties (Bulk modulus, Young's modulus, Shear modulus, Poisson's ratio) according to Voigt, Reuss, Hill.
    Smat = calc_Smat(EM)

    Voigt, Reuss, Hill = averages(EM)
    KV, GV, EV, nuV = Voigt
    KR, GR, ER, nuR = Reuss
    KH, GH, EH, nuH = Hill
    resdata['average_properties'] = {'Voigt': Voigt, 'Reuss': Reuss, 'Hill': Hill}

    # Print results
    txt2output = txt2output + "Average properties (Bulk modulus (K), Young's modulus (E), Shear modulus (G), Poisson's ratio (ν)):" + '\n'
    txt2output = txt2output + "Method:    K(GPa)\tE(GPa)\tG(GPa)\tν" + '\n'
    txt2output = txt2output + f'Voigt:    {KV:.3f}\t{EV:.3f}\t{GV:.3f}\t{nuV:.3f}' + '\n'
    txt2output = txt2output + f'Reuss:    {KR:.3f}\t{ER:.3f}\t{GR:.3f}\t{nuR:.3f}' + '\n'
    txt2output = txt2output + f'Hill:     {KH:.3f}\t{EH:.3f}\t{GH:.3f}\t{nuH:.3f}' + '\n'
    txt2output = txt2output + f'Universal anisotropy index (A^U): {KV/KR+5*GV/GR -6:.4f}'+'\n'
    txt2output = txt2output + f'G/K: {GH/KH:.5f}' + '\n'
    txt2output = txt2output + '\n'

    minf_Y, maxf_Y, cd_min_Y, cd_max_Y = get_min_max_directions(Smat,'Young_tp')
    anis_Y = minf_Y/maxf_Y
    minf_LC, maxf_LC, cd_min_LC, cd_max_LC = get_min_max_directions(Smat,'LinearCompressibility_tp')
    anis_LC = minf_LC/maxf_LC
    minf_S0, maxf_S0, cd_min_S0, cd_max_S0 = get_min_max_directions(Smat,'shear2D', 0)
    anis_S0 = minf_S0/maxf_S0
    minf_S1, maxf_S1, cd_min_S1, cd_max_S1 = get_min_max_directions(Smat,'shear2D', 1)
    anis_S1 = minf_S1/maxf_S1
    minf_P1, maxf_P1, cd_min_P1, cd_max_P1 = get_min_max_directions(Smat,'Poisson2D', 1)
    anis_P1 = minf_P1/maxf_P1
    minf_P2, maxf_P2, cd_min_P2, cd_max_P2 = get_min_max_directions(Smat,'Poisson2D', 2)
    anis_P2 = minf_P2/maxf_P2

    resdata['variation_properties'] = {'Young': {'min': minf_Y, 'max': maxf_Y, 'anisotropy': anis_Y, 'min_direction': cd_min_Y, 'max_direction': cd_max_Y},
                                        'LinearCompressibility': {'min': minf_LC, 'max': maxf_LC, 'anisotropy': anis_LC, 'min_direction': cd_min_LC, 'max_direction': cd_max_LC},
                                        'Shear': {'min': minf_S0, 'max': maxf_S0, 'anisotropy': anis_S0, 'min_direction': cd_min_S0, 'max_direction': cd_max_S0},
                                        'Shear2': {'min': minf_S1, 'max': maxf_S1, 'anisotropy': anis_S1, 'min_direction': cd_min_S1, 'max_direction': cd_max_S1},
                                        'Poisson': {'min': minf_P1, 'max': maxf_P1, 'anisotropy': anis_P1, 'min_direction': cd_min_P1, 'max_direction': cd_max_P1},
                                        'Poisson2': {'min': minf_P2, 'max': maxf_P2, 'anisotropy': anis_P2, 'min_direction': cd_min_P2, 'max_direction': cd_max_P2}
                                        }

    txt2output = txt2output +  "Variations of the elastic moduli:" + '\n'
    txt2output = txt2output +  "           Young's mod. | Lin. comp. | Shear mod.| Poisson's ratio" + '\n'
    txt2output = txt2output +  '          --------------|------------|-----------|------------------' + '\n'
    txt2output = txt2output + f'Min:       {minf_Y:.3f}\t|  {minf_LC:.3f}     |\t{minf_S0:.3f}\t |\t{minf_P1:.3f}' + '\n'
    txt2output = txt2output + f'                        |            |\t{minf_S1:.3f}\t |\t{minf_P2:.3f}' + '\n'
    txt2output = txt2output +  '          --------------|------------|-----------|------------------' + '\n'
    txt2output = txt2output + f'Max:       \t{maxf_Y:.3f}\t|  {maxf_LC:.3f}     |\t{maxf_S0:.3f}\t |\t{maxf_P1:.3f}' + '\n'
    txt2output = txt2output + f'            \t\t|            |\t{maxf_S1:.3f}\t |\t{maxf_P2:.3f}' + '\n'
    txt2output = txt2output + '          --------------|------------|-----------|------------------' + '\n'
    txt2output = txt2output + f'Anisotropy:\t{anis_Y:.3f}\t|  {anis_LC:.3f}     |\t{anis_S0:.3f}\t |\t{anis_P1:.3f}'   + '\n'
    txt2output = txt2output + f'            \t\t|            |\t{anis_S1:.3f}\t |\t{anis_P2:.3f}' + '\n'
    txt2output = txt2output + '          --------------|------------|-----------|------------------' + '\n'
    txt2output = txt2output + f'direction (min): {cd_min_Y[0]:.3f}\t|  {cd_min_LC[0]:.3f}     |  {cd_min_S0[0]:.3f}\t |\t{cd_min_P1[0]:.3f}' + '\n'
    txt2output = txt2output + f'                 {cd_min_Y[1]:.3f}\t|  {cd_min_LC[1]:.3f}     |  {cd_min_S0[1]:.3f}\t |\t{cd_min_P1[1]:.3f}' + '\n'
    txt2output = txt2output + f'                 {cd_min_Y[2]:.3f}\t|  {cd_min_LC[2]:.3f}     |  {cd_min_S0[2]:.3f}\t |\t{cd_min_P1[2]:.3f}' + '\n'
    txt2output = txt2output + f'                        |            |  {cd_min_S1[0]:.3f}\t |\t{cd_min_P2[0]:.3f}' + '\n'
    txt2output = txt2output + f'                        |            |  {cd_min_S1[1]:.3f}\t |\t{cd_min_P2[1]:.3f}' + '\n'
    txt2output = txt2output + f'                        |            |  {cd_min_S1[2]:.3f}\t |\t{cd_min_P2[2]:.3f}' + '\n'
    txt2output = txt2output + '          --------------|------------|-----------|------------------' + '\n'
    txt2output = txt2output + f'direction (max): {cd_max_Y[0]:.3f}\t|  {cd_max_LC[0]:.3f}     |  {cd_max_S0[0]:.3f}\t |\t{cd_max_P1[0]:.3f}' + '\n'
    txt2output = txt2output + f'                 {cd_max_Y[1]:.3f}\t|  {cd_max_LC[1]:.3f}     |  {cd_max_S0[1]:.3f}\t |\t{cd_max_P1[1]:.3f}' + '\n'
    txt2output = txt2output + f'                 {cd_max_Y[2]:.3f}\t|  {cd_max_LC[2]:.3f}     |  {cd_max_S0[2]:.3f}\t |\t{cd_max_P1[2]:.3f}' + '\n'
    txt2output = txt2output + f'                        |            |  {cd_max_S1[0]:.3f}\t |\t{cd_max_P2[0]:.3f}' + '\n'
    txt2output = txt2output + f'                        |            |  {cd_max_S1[1]:.3f}\t |\t{cd_max_P2[1]:.3f}' + '\n'
    txt2output = txt2output + f'                        |            |  {cd_max_S1[2]:.3f}\t |\t{cd_max_P2[2]:.3f}' + '\n'
    txt2output = txt2output + '\n'


    return txt2output, resdata


def run_script_plots(axs, EM):
    Smat = calc_Smat(EM)
    axY, axL, axS, axP = axs

    Young_funct = lambda t, p: Young_tp(t, p, Smat)
    LinearCompressibility_funct = lambda t, p: LinearCompressibility_tp(t, p, Smat)
    Shear2D_funct = lambda t, p: shear2D([t, p], Smat)
    Poisson2D_funct = lambda t, p: Poisson2D([t, p], Smat)

    phi = np.linspace(0, 2 * np.pi, 100)
    theta = np.linspace(0, 2*np.pi, 100)

    fY = np.vectorize(Young_funct)
    fL = np.vectorize(LinearCompressibility_funct)
    fS = np.vectorize(Shear2D_funct)
    fP = np.vectorize(Poisson2D_funct)

    r_xyY = fY(np.pi / 2, phi)
    r_xzY = fY(-(theta+np.pi/2), 0)
    r_yzY = fY(-(theta+np.pi/2), np.pi/2)

    r_xyL = fL(np.pi / 2, phi)
    r_xzL = fL(-(theta+np.pi/2), 0)
    r_yzL = fL(-(theta+np.pi/2), np.pi/2)

    r_xyS = fS(np.pi/2, phi)
    r_xzS = fS(theta + np.pi/2, np.pi)
    r_yzS = fS(-(theta + np.pi/2), np.pi/2)

    r_xyP = fP(np.pi/2, phi)
    r_xzP = fP(theta + np.pi/2, np.pi)
    r_yzP = fP(-(theta + np.pi/2), np.pi/2)

    axY[0].plot(phi, r_xyY)
    axY[1].plot(phi, r_xzY)
    axY[2].plot(phi, r_yzY)

    axL[0].plot(phi, r_xyL)
    axL[1].plot(phi, r_xzL)
    axL[2].plot(phi, r_yzL)

    axS[0].plot(phi, r_xyS[0], color='b')
    axS[0].plot(phi, r_xyS[1], color='r')

    axS[1].plot(theta, r_xzS[0], color='b')
    axS[1].plot(theta, r_xzS[1], color='r')

    axS[2].plot(theta, r_yzS[0], color='b')
    axS[2].plot(theta, r_yzS[1], color='r')

    axP[0].plot(phi, r_xyP[0], color='g')
    axP[0].plot(phi, r_xyP[1], color='r')
    axP[0].plot(phi, r_xyP[2], color='b')

    axP[1].plot(theta, r_xzP[0], color='g')
    axP[1].plot(theta, r_xzP[1], color='r')
    axP[1].plot(phi, r_xzP[2], color='b')

    axP[2].plot(theta, r_yzP[0], color='g')
    axP[2].plot(theta, r_yzP[1], color='r')
    axP[2].plot(theta, r_yzP[2], color='b')

    axY[0].text(np.pi*1.1, axY[0].get_rmax() * 2.2, r'$E(\theta, \phi)~[GPa]$', rotation=90)
    axL[0].text(np.pi*1.1, axL[0].get_rmax() * 2.2, r'$\beta(\theta, \phi)$', rotation=90)
    axS[0].text(np.pi*1.1, axS[0].get_rmax() * 2.2, r'$G(\theta, \phi, \chi)~[GPa]$', rotation=90)
    axP[0].text(np.pi*1.1, axP[0].get_rmax() * 2.2, r'$\nu(\theta, \phi, \chi)$', rotation=90)

    # axY[0].set_title('x-y plane')
    # axY[1].set_title('x-z plane')
    # axY[2].set_title('y-z plane')

    for ax in [axY, axL, axS, axP]:
        for i in range(3):
            ax[i].tick_params(axis='y', labelsize=8, labelrotation=292.5)
    # plt.show()

    return axs






import re
import traceback

import numpy as np
from PySide6.QtGui import QIcon
from PySide6.QtWidgets import QMainWindow, QMessageBox
from debyetools.poisson import poisson_ratio as dt_poisson_ratio

from debyetools.tpropsgui.atomtools import dt_potentials, Molecule
from debyetools.tpropsgui.cp_window import dialogCpWindow
from debyetools.tpropsgui.ui_main_window_small import Ui_MainWindow

from debyetools.tpropsgui.dialog_fitEOS import dialogFitEOS
from debyetools.tpropsgui.dialog_calculateNu import dialogCalcNu
from debyetools.tpropsgui.dialog_doscar import dialogDoscar
from PySide6.QtCore import QTimer

from PySide6.QtGui import QPixmap, QPalette
from matplotlib import pyplot as plt
import sys
import io

from PySide6.QtCore import QByteArray, QBuffer
import base64

def pixmap_to_base64(pixmap):
    # Convert QPixmap to QByteArray
    byte_array = QByteArray()
    buffer = QBuffer(byte_array)
    buffer.open(QBuffer.WriteOnly)
    pixmap.save(buffer, "PNG")  # Save the QPixmap to buffer as PNG
    buffer.close()
    # Encode QByteArray to base64 and decode to string for HTML embedding
    return byte_array.toBase64().data().decode()

# Updated `create_latex_image` function as before
def create_latex_image(latex_str, color):

    plt.figure(figsize=(2, 1))
    plt.text(0.1, 0.1, f"{latex_str}", fontsize=12, ha='left', va='center', c=color)
    plt.axis('off')
    buffer = io.BytesIO()
    plt.savefig(buffer, format='png', bbox_inches='tight', pad_inches=0.1, transparent=True)
    buffer.seek(0)

    pixmap = QPixmap()
    success = pixmap.loadFromData(buffer.getvalue())
    if not success:
        raise ValueError("Failed to load LaTeX image into QPixmap.")
    return pixmap


def highlight_line_edit(line_edit, color="purple", duration=100):
    # Set the background color
    line_edit.setStyleSheet(f"background-color: {color};")

    # Create a QTimer to reset the color after `duration` milliseconds
    timer = QTimer(line_edit)
    timer.setSingleShot(True)  # Only trigger once
    timer.timeout.connect(lambda: line_edit.setStyleSheet(""))  # Reset the color
    timer.start(duration)

def separate_atoms(formula):
    # This pattern matches element symbols (one or two letters, starting with an uppercase letter)
    # followed by optional counts (numbers)
    pattern = r"([A-Z][a-z]*)(\d*)"
    elements = re.findall(pattern, formula)
    separated = []

    for element, count in elements:
        # If count is empty, it means there is one atom of this element
        count = int(count) if count else 1
        separated.extend([element] * count)

    return separated


class dialogMainWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        my_icon = QIcon()
        my_icon.addFile('icon.ico')

        self.setWindowIcon(my_icon)

        self.cp_window = dialogCpWindow(self)

        self.ui.pushButton_5.clicked.connect(self.on_pushButton_Cp)
        self.ui.pushgoback.clicked.connect(self.on_pushButton_goback)

        self.eos_str = 'BM'

        self.molecule = Molecule()

        self.ui.comboBox.currentIndexChanged.connect(self.selectionchange)

        self.check_el, self.check_def, self.check_anh, self.check_xs = self.ui.checkBox, self.ui.checkBox_2, self.ui.checkBox_3, self.ui.checkBox_4
        self.check_xspol = self.ui.checkBox_5

        self.state_el = False
        self.state_def = False
        self.state_anh = False
        self.state_xs = False
        self.state_xspol = False

        self.check_el.stateChanged.connect(self.on_check_el)
        self.check_def.stateChanged.connect(self.on_check_def)
        self.check_anh.stateChanged.connect(self.on_check_anh)
        self.check_xs.stateChanged.connect(self.on_check_xs)
        self.check_xspol.stateChanged.connect(self.on_check_xspol)

        self.dialogFitEOS = dialogFitEOS(self)
        self.dialogCalcNu = dialogCalcNu(self)

        self.ui.pushFitEOS.clicked.connect(self.on_pushFitEOS)

        self.ui.pushNu.clicked.connect(self.on_pushNu)

        self.ui.pushDoscar.clicked.connect(self.on_pushDoscar)

        self.dialogDoscar = dialogDoscar(self)

        self.ui.lineEdit.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit))
        self.ui.lineEdit_2.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_2))
        self.ui.lineEdit_3.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_3))
        self.ui.lineEdit_11.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_11))
        self.ui.lineEdit_el.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_el))
        self.ui.lineEdit_def.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_def))
        self.ui.lineEdit_anh.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_anh))
        self.ui.lineEdit_xs.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_xs))
        self.ui.lineEdit_xspol.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_xspol))
        self.ui.lineEdit_T.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_T))
        self.ui.lineEdit_P.textChanged.connect(lambda: self.on_text_changed(self.ui.lineEdit_P))

        # Generate LaTeX image and display it in tooltip
        # Usage in the main code
        latex_string = '- Birch-Murnaghan:\n' + r"    $E(V) = \left(-\frac{9}{16} V_0 B_0 B'_0 + \frac{27}{8} V_0 B_0 + E_0\right) + \frac{\frac{27}{16} V_0^{5/3} B_0 B'_0 - 9 V_0^{5/3} B_0}{V^{2/3}} + \frac{-\frac{27}{16} V_0^{7/3} B_0 B'_0 + \frac{63}{8} V_0^{7/3} B_0}{V^{4/3}} + \frac{\frac{9}{16} V_0^3 B_0 B'_0 - \frac{9}{4} V_0^3 B_0}{V^2}$" + \
        '\n- Rose-Vinet:\n' + r"    $E(V) = E_0 - \frac{2 B_0 V_0 \exp\left(-\frac{3}{2}(B'_0 - 1) \left(-1 + \left(\frac{V}{V_0}\right)^{\frac{1}{3}}\right)\right)}{(B'_0 - 1)^2} \left( 3 \left(\frac{V}{V_0}\right)^{\frac{1}{3}} B'_0 - 3 B'_0 - 3 \left(\frac{V}{V_0}\right)^{\frac{1}{3}} + 5 \right) + \frac{4 B_0 V_0}{(B'_0 - 1)^2}$" + \
        '\n- Mie-Gruneisen:\n' + r"    $E(V) = \frac{9 \left( 3 \left( B_0 V_0 + \frac{1}{3} B'_0 E_0 - \frac{7}{9} E_0 \right) \left( B'_0 - \frac{8}{3} \right) \left( \frac{V}{V_0} \right)^{\frac{1}{3}} + B_0 V_0 \left( \left( \frac{V}{V_0} \right)^{\frac{8}{3} - B'_0} - 3 B'_0 + 7 \right) \right)}{\left( \frac{V}{V_0} \right)^{\frac{1}{3}} \left( 9 {B'_0}^2 - 45 B'_0 + 56 \right)}$"+\
        '\n- TB-SMA:\n' + r"    $E(V) = -\frac{X_x \exp\left(3 B'_0 - 3 - \frac{X_x}{2 E_0}\right)}{2 \left(3 B'_0 - 3 - \frac{X_x}{E_0}\right)} \cdot \exp\left(-\frac{3 B'_0 - 3 - \frac{X_x}{2 E_0}}{V_0^{1/3}} V^{1/3}\right) + \frac{E_0 \left(3 B'_0 - 3 - \frac{X_x}{2 E_0}\right) \exp\left(\frac{X_x}{2 E_0}\right)}{3 B'_0 - 3 - \frac{X_x}{E_0}} \cdot \exp\left(-\frac{X_x}{2 E_0 V_0^{1/3}} V^{1/3}\right)$" + \
        '\n- Murnaghan:\n' + r"    $E(V) = E_0 + \frac{B_0 V_0}{B'_0 (B'_0 - 1)} \left( \frac{V}{V_0} \right)^{1 - B'_0} + \frac{B_0}{B'_0} V - \frac{B_0 V_0}{B'_0 - 1}$" + \
        '\n- Poirier-Tarantola:\n' + r"    $E(V) = E_0 + \frac{1}{2} B_0 V_0 \left( \ln \frac{V}{V_0} \right)^2 - \frac{1}{6} B_0 V_0 \left( B'_0 - 2 \right) \left( \ln \frac{V}{V_0} \right)^3$" + \
        '\n- Morse interatomic potential:\n' + r"    $E\left(V\right) = \sum_{ij}D_i\cdot\left(2e^{2\alpha_i (r_j-r^0_{ij})} - e^{-\alpha (r_j-r^0_{ij})}\right)$" + \
        '\n- Embeded-Atom model interatomic potential:\n' + r"    $E(V)=\sum_{i}F_{i}\left(\rho_{h,i}\right)+\frac{1}{2}\sum_{i\neq j}\phi_{ij}\left(R_{ij}\right)$"


        # Check if the system is in dark mode by detecting the background color
        dark_mode = self.is_dark_mode()
        color = "white" if dark_mode else "black"
        latex_pixmap = create_latex_image(latex_string, color)
        base64_image = pixmap_to_base64(latex_pixmap)
        tooltip_html = f'<img src="data:image/png;base64,{base64_image}">'
        self.ui.moreinfo.setToolTip(tooltip_html)

    def is_dark_mode(self):
        # Detect if the application is in dark mode using the palette
        palette = self.palette()
        return palette.color(QPalette.ColorRole.Window).value() < 128  # Lightness threshold for dark mode


    def selectionchange(self, i):
        # self.ui.progress_2.setValue(0)
        dict_eos = {'Birch-Murnaghan': 'BM', 'Rose-Vinet': 'RV', 'Mie-Gruneisen': 'MG', 'TB-SMA': 'TB',
                    'Murnaghan': 'MU', 'Poirier-Tarantola': 'PT', 'Morse potential': 'MP',
                    'EAM int. potential': 'EAM'}
        self.eos_str = dict_eos[self.ui.comboBox.itemText(i)]

        self.ui.lineEdit_2.setText('-3e5, 1e-5, 7e10, 4')

    def get_C(self):
        txt = self.ui.elastic_constants.toPlainText().replace('XX', ' ').replace('YY', ' ').replace('ZZ', ' ').replace(
            'XY', ' ').replace('YZ', ' ').replace('ZX', ' ').split('\n')
        data_lst = []
        for ti in txt:
            if len(ti) == 0: continue
            if ti[0] == '#': continue
            data_lst.append([float(tii) for tii in ti.replace('\t', ' ').split()])
        data = np.array(data_lst)

        return data

    def on_check_el(self):
        self.state_el = self.check_el.isChecked()
        self.ui.lineEdit_el.setEnabled(self.state_el)

    def on_check_def(self):
        self.state_def = self.check_def.isChecked()
        self.ui.lineEdit_def.setEnabled(self.state_def)

    def on_check_anh(self):
        self.state_anh = self.check_anh.isChecked()
        self.ui.lineEdit_anh.setEnabled(self.state_anh)

    def on_check_xs(self):
        self.state_xs = self.check_xs.isChecked()
        self.ui.lineEdit_xs.setEnabled(self.state_xs)

    def on_check_xspol(self):
        self.state_xspol = self.check_xspol.isChecked()
        self.ui.lineEdit_xspol.setEnabled(self.state_xspol)


    def on_pushButton_goback(self):
        self.close()


    def on_pushNu(self):
        self.dialogCalcNu.external_nu = self.ui.lineEdit_3
        self.dialogCalcNu.show()

    def on_pushFitEOS(self):
        # print(self.ui.lineEdit_2.text())
        self.dialogFitEOS.external_combobox = self.ui.comboBox
        self.dialogFitEOS.external_EOS_txt = self.ui.lineEdit_2
        self.dialogFitEOS.ui.lineEdit_3.setText(self.ui.lineEdit_2.text())
        list_items_in_combox = [self.ui.comboBox.itemText(i) for i in range(self.ui.comboBox.count())]
        # print(list_items_in_combox)

        self.dialogFitEOS.ui.comboBox_2.clear()
        for item_cb in list_items_in_combox:
            self.dialogFitEOS.ui.comboBox_2.addItem(item_cb)


        # print('CI', self.ui.comboBox.currentIndex())
        self.dialogFitEOS.ui.comboBox_2.setCurrentText(list_items_in_combox[self.ui.comboBox.currentIndex()])
        self.dialogFitEOS.molecule = self.molecule
        if self.eos_str == 'MP':
            self.dialogFitEOS.molecule_from_crystal = self.molecule_from_crystal



        self.dialogFitEOS.show()



    def on_pushButton_Cp(self):
        self.cp_window.app=self.app

        self.was_Cp_calculated = True
        pattern = r'[A-Z][a-z]?'
        error_msg = ''
        try:
            formula_comp = self.ui.lineEdit_11.text()
            self.molecule.r = len(set(re.findall(pattern, formula_comp)))
            self.molecule.nu = float(self.ui.lineEdit_3.text())
            self.molecule.mass = float(self.ui.lineEdit.text())

            self.molecule.p_anh = [float(si) for si in
                                   self.ui.lineEdit_anh.text().replace(',', ' ').split()] if self.state_anh else [0, 1]
            self.molecule.p_el = [float(si) for si in
                                  self.ui.lineEdit_el.text().replace(',', ' ').split()] if self.state_el else [0, 0, 0,
                                                                                                               0]
            self.molecule.p_def = [float(si) for si in
                                   self.ui.lineEdit_def.text().replace(',', ' ').split()] if self.state_def else [100,
                                                                                                                  1,
                                                                                                                  1000,
                                                                                                                  0.1]
            self.molecule.p_xs = [float(si) for si in
                                  self.ui.lineEdit_xs.text().replace(',', ' ').split()] if self.state_xs else [0, 0, 0]

            self.molecule.initial_params = [float(si) for si in self.ui.lineEdit_2.text().replace(',', ' ').split()]

            self.molecule.xsparams = [float(si) for si in
            self.ui.lineEdit_xspol.text().replace(',', ' ').split()] if self.state_xs else [0, 0, 0, 0, 0, 0]

            if self.eos_str in ['MP', 'EAM']:

                for karg in ['cell', 'basis', 'types', 'formula', 'number_of_NNs', 'cutoff', 'distances', 'num_bonds_per_formula', 'combs_types']:
                    setattr(self.molecule, karg, getattr(self.molecule_from_crystal, karg))

                args = [self.molecule.formula, self.molecule.cell, self.molecule.basis, self.molecule.cutoff,
                        self.molecule.number_of_NNs]
            else:
                args = [None]
            self.molecule.set_eos(self.eos_str, args)

            self.cp_window.ui.lineEdit.setText(self.ui.lineEdit_T.text())
            self.cp_window.ui.lineEdit_2.setText(self.ui.lineEdit_P.text())

            types = separate_atoms(self.ui.lineEdit_11.text())
            self.molecule.update_fomula(types)
            self.cp_window.debye_run(self.molecule, self.ui.progress, self.ui.lineEdit_11.text())
            self.cp_window.show()
        except Exception as e:
            print(e)
            error_message = f"An error occurred: {e}\n"
            error_traceback = traceback.format_exc()

            # Combine the error message with the traceback
            full_error_text = error_message + error_traceback
            print(full_error_text)


            if 'self.molecule.nu' in full_error_text:
                error_msg = "Something is wrong with the Poisson's ratio."
            elif 'self.molecule.mass' in full_error_text:
                error_msg = "Something is wrong with the mass."
            elif 'E0, V0, B0, Bp0 = pEOS' in full_error_text:
                error_msg = "Something is wrong with the EOS's parameters."
            elif 'q0, q1, q2, q3 = p_electronic' in full_error_text:
                error_msg = "Something is wrong with the parameters for the electronic contribution."
            elif 'self.molecule.p_el' in full_error_text:
                error_msg = "Something is wrong with the parameters for the electronic contribution."

            if error_msg == '':
                error_msg = full_error_text
            QMessageBox.information(self, 'Error', error_msg, QMessageBox.Ok)

    def on_pushDoscar(self):
        status = 0
        try:
            self.dialogDoscar.Vdata = self.dialogFitEOS.Vdata
            status =1
        except AttributeError:
            QMessageBox.information(self, 'Warning', 'The volume data was not loaded!\n Please make sure you run the EOS fitting first.', QMessageBox.Ok)
        if status == 1:
            self.dialogDoscar.external_iparams = self.ui.lineEdit_el
            self.dialogDoscar.show()

    def on_text_changed(self, line_edit):
        # Call the reusable highlight function
        dark_mode = self.is_dark_mode()
        color_p = "purple" if dark_mode else "#94a2c9"

        highlight_line_edit(line_edit, color=color_p, duration=500)





# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'dialog_crystal.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QHBoxLayout, QHeaderView,
    QLabel, QLineEdit, QPlainTextEdit, QPushButton,
    QSizePolicy, QSpacerItem, QTableWidget, QTableWidgetItem,
    QVBoxLayout, QWidget)

class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.setWindowModality(Qt.WindowModality.WindowModal)
        Form.resize(779, 422)
        self.horizontalLayout = QHBoxLayout(Form)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.label = QLabel(Form)
        self.label.setObjectName(u"label")

        self.verticalLayout.addWidget(self.label)

        self.tableCell = QTableWidget(Form)
        if (self.tableCell.columnCount() < 3):
            self.tableCell.setColumnCount(3)
        __qtablewidgetitem = QTableWidgetItem()
        self.tableCell.setHorizontalHeaderItem(0, __qtablewidgetitem)
        __qtablewidgetitem1 = QTableWidgetItem()
        self.tableCell.setHorizontalHeaderItem(1, __qtablewidgetitem1)
        __qtablewidgetitem2 = QTableWidgetItem()
        self.tableCell.setHorizontalHeaderItem(2, __qtablewidgetitem2)
        if (self.tableCell.rowCount() < 3):
            self.tableCell.setRowCount(3)
        __qtablewidgetitem3 = QTableWidgetItem()
        self.tableCell.setVerticalHeaderItem(0, __qtablewidgetitem3)
        __qtablewidgetitem4 = QTableWidgetItem()
        self.tableCell.setVerticalHeaderItem(1, __qtablewidgetitem4)
        __qtablewidgetitem5 = QTableWidgetItem()
        self.tableCell.setVerticalHeaderItem(2, __qtablewidgetitem5)
        __qtablewidgetitem6 = QTableWidgetItem()
        self.tableCell.setItem(0, 0, __qtablewidgetitem6)
        __qtablewidgetitem7 = QTableWidgetItem()
        self.tableCell.setItem(0, 1, __qtablewidgetitem7)
        __qtablewidgetitem8 = QTableWidgetItem()
        self.tableCell.setItem(0, 2, __qtablewidgetitem8)
        __qtablewidgetitem9 = QTableWidgetItem()
        self.tableCell.setItem(1, 0, __qtablewidgetitem9)
        __qtablewidgetitem10 = QTableWidgetItem()
        self.tableCell.setItem(1, 1, __qtablewidgetitem10)
        __qtablewidgetitem11 = QTableWidgetItem()
        self.tableCell.setItem(1, 2, __qtablewidgetitem11)
        __qtablewidgetitem12 = QTableWidgetItem()
        self.tableCell.setItem(2, 0, __qtablewidgetitem12)
        __qtablewidgetitem13 = QTableWidgetItem()
        self.tableCell.setItem(2, 1, __qtablewidgetitem13)
        __qtablewidgetitem14 = QTableWidgetItem()
        self.tableCell.setItem(2, 2, __qtablewidgetitem14)
        self.tableCell.setObjectName(u"tableCell")
        self.tableCell.setMinimumSize(QSize(165, 100))
        self.tableCell.setMaximumSize(QSize(165, 100))
        self.tableCell.setFrameShape(QFrame.Shape.Box)
        self.tableCell.horizontalHeader().setVisible(False)
        self.tableCell.horizontalHeader().setDefaultSectionSize(50)
        self.tableCell.verticalHeader().setDefaultSectionSize(25)

        self.verticalLayout.addWidget(self.tableCell)

        self.label_2 = QLabel(Form)
        self.label_2.setObjectName(u"label_2")

        self.verticalLayout.addWidget(self.label_2)

        self.tableBasis = QTableWidget(Form)
        if (self.tableBasis.columnCount() < 4):
            self.tableBasis.setColumnCount(4)
        __qtablewidgetitem15 = QTableWidgetItem()
        self.tableBasis.setHorizontalHeaderItem(0, __qtablewidgetitem15)
        __qtablewidgetitem16 = QTableWidgetItem()
        self.tableBasis.setHorizontalHeaderItem(1, __qtablewidgetitem16)
        __qtablewidgetitem17 = QTableWidgetItem()
        self.tableBasis.setHorizontalHeaderItem(2, __qtablewidgetitem17)
        __qtablewidgetitem18 = QTableWidgetItem()
        self.tableBasis.setHorizontalHeaderItem(3, __qtablewidgetitem18)
        if (self.tableBasis.rowCount() < 10):
            self.tableBasis.setRowCount(10)
        __qtablewidgetitem19 = QTableWidgetItem()
        self.tableBasis.setVerticalHeaderItem(0, __qtablewidgetitem19)
        __qtablewidgetitem20 = QTableWidgetItem()
        self.tableBasis.setVerticalHeaderItem(1, __qtablewidgetitem20)
        __qtablewidgetitem21 = QTableWidgetItem()
        self.tableBasis.setVerticalHeaderItem(2, __qtablewidgetitem21)
        __qtablewidgetitem22 = QTableWidgetItem()
        self.tableBasis.setVerticalHeaderItem(3, __qtablewidgetitem22)
        __qtablewidgetitem23 = QTableWidgetItem()
        self.tableBasis.setVerticalHeaderItem(4, __qtablewidgetitem23)
        __qtablewidgetitem24 = QTableWidgetItem()
        self.tableBasis.setVerticalHeaderItem(5, __qtablewidgetitem24)
        __qtablewidgetitem25 = QTableWidgetItem()
        self.tableBasis.setVerticalHeaderItem(6, __qtablewidgetitem25)
        __qtablewidgetitem26 = QTableWidgetItem()
        self.tableBasis.setVerticalHeaderItem(7, __qtablewidgetitem26)
        __qtablewidgetitem27 = QTableWidgetItem()
        self.tableBasis.setVerticalHeaderItem(8, __qtablewidgetitem27)
        __qtablewidgetitem28 = QTableWidgetItem()
        self.tableBasis.setVerticalHeaderItem(9, __qtablewidgetitem28)
        __qtablewidgetitem29 = QTableWidgetItem()
        self.tableBasis.setItem(0, 0, __qtablewidgetitem29)
        __qtablewidgetitem30 = QTableWidgetItem()
        self.tableBasis.setItem(0, 1, __qtablewidgetitem30)
        __qtablewidgetitem31 = QTableWidgetItem()
        self.tableBasis.setItem(0, 2, __qtablewidgetitem31)
        __qtablewidgetitem32 = QTableWidgetItem()
        self.tableBasis.setItem(0, 3, __qtablewidgetitem32)
        __qtablewidgetitem33 = QTableWidgetItem()
        self.tableBasis.setItem(1, 0, __qtablewidgetitem33)
        __qtablewidgetitem34 = QTableWidgetItem()
        self.tableBasis.setItem(1, 1, __qtablewidgetitem34)
        __qtablewidgetitem35 = QTableWidgetItem()
        self.tableBasis.setItem(1, 2, __qtablewidgetitem35)
        __qtablewidgetitem36 = QTableWidgetItem()
        self.tableBasis.setItem(1, 3, __qtablewidgetitem36)
        __qtablewidgetitem37 = QTableWidgetItem()
        self.tableBasis.setItem(2, 0, __qtablewidgetitem37)
        __qtablewidgetitem38 = QTableWidgetItem()
        self.tableBasis.setItem(2, 1, __qtablewidgetitem38)
        __qtablewidgetitem39 = QTableWidgetItem()
        self.tableBasis.setItem(2, 2, __qtablewidgetitem39)
        __qtablewidgetitem40 = QTableWidgetItem()
        self.tableBasis.setItem(2, 3, __qtablewidgetitem40)
        __qtablewidgetitem41 = QTableWidgetItem()
        self.tableBasis.setItem(3, 0, __qtablewidgetitem41)
        __qtablewidgetitem42 = QTableWidgetItem()
        self.tableBasis.setItem(3, 1, __qtablewidgetitem42)
        __qtablewidgetitem43 = QTableWidgetItem()
        self.tableBasis.setItem(3, 2, __qtablewidgetitem43)
        __qtablewidgetitem44 = QTableWidgetItem()
        self.tableBasis.setItem(3, 3, __qtablewidgetitem44)
        self.tableBasis.setObjectName(u"tableBasis")
        self.tableBasis.setMaximumSize(QSize(250, 200))
        self.tableBasis.setFrameShape(QFrame.Shape.Box)
        self.tableBasis.horizontalHeader().setDefaultSectionSize(40)
        self.tableBasis.verticalHeader().setDefaultSectionSize(25)

        self.verticalLayout.addWidget(self.tableBasis)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalLayout_3.setContentsMargins(-1, 5, -1, 5)
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(5, 5, 5, 5)
        self.label_3 = QLabel(Form)
        self.label_3.setObjectName(u"label_3")

        self.verticalLayout_2.addWidget(self.label_3)

        self.label_4 = QLabel(Form)
        self.label_4.setObjectName(u"label_4")

        self.verticalLayout_2.addWidget(self.label_4)

        self.label_5 = QLabel(Form)
        self.label_5.setObjectName(u"label_5")

        self.verticalLayout_2.addWidget(self.label_5)


        self.horizontalLayout_3.addLayout(self.verticalLayout_2)

        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(5, 5, 5, 5)
        self.lineEditCutoff = QLineEdit(Form)
        self.lineEditCutoff.setObjectName(u"lineEditCutoff")
        self.lineEditCutoff.setMaximumSize(QSize(70, 16777215))

        self.verticalLayout_3.addWidget(self.lineEditCutoff)

        self.lineEditNnn = QLineEdit(Form)
        self.lineEditNnn.setObjectName(u"lineEditNnn")
        self.lineEditNnn.setMaximumSize(QSize(70, 16777215))

        self.verticalLayout_3.addWidget(self.lineEditNnn)

        self.lineEditInitialGuess = QLineEdit(Form)
        self.lineEditInitialGuess.setObjectName(u"lineEditInitialGuess")
        self.lineEditInitialGuess.setMaximumSize(QSize(70, 16777215))

        self.verticalLayout_3.addWidget(self.lineEditInitialGuess)


        self.horizontalLayout_3.addLayout(self.verticalLayout_3)

        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.verticalLayout_4.setContentsMargins(5, 5, 5, 5)
        self.label_7 = QLabel(Form)
        self.label_7.setObjectName(u"label_7")

        self.verticalLayout_4.addWidget(self.label_7)

        self.label_6 = QLabel(Form)
        self.label_6.setObjectName(u"label_6")

        self.verticalLayout_4.addWidget(self.label_6)

        self.label_8 = QLabel(Form)
        self.label_8.setObjectName(u"label_8")

        self.verticalLayout_4.addWidget(self.label_8)


        self.horizontalLayout_3.addLayout(self.verticalLayout_4)


        self.verticalLayout.addLayout(self.horizontalLayout_3)


        self.horizontalLayout.addLayout(self.verticalLayout)

        self.verticalLayout_5 = QVBoxLayout()
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.verticalLayout_5.setContentsMargins(5, 5, 5, 5)
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_xxxx = QHBoxLayout()
        self.horizontalLayout_xxxx.setObjectName(u"horizontalLayout_xxxx")
        self.horizontalLayout_xxxx.setContentsMargins(10, 10, 10, -1)

        self.horizontalLayout_2.addLayout(self.horizontalLayout_xxxx)

        self.plainPairAnalysis = QPlainTextEdit(Form)
        self.plainPairAnalysis.setObjectName(u"plainPairAnalysis")
        self.plainPairAnalysis.setEnabled(True)
        self.plainPairAnalysis.setMinimumSize(QSize(150, 150))
        self.plainPairAnalysis.setMaximumSize(QSize(175, 400))
        palette = QPalette()
        self.plainPairAnalysis.setPalette(palette)
        font = QFont()
        font.setFamilies([u"Cascadia Mono"])
        font.setPointSize(8)
        self.plainPairAnalysis.setFont(font)
        self.plainPairAnalysis.setFrameShape(QFrame.Shape.Box)
        self.plainPairAnalysis.setTabChangesFocus(False)
        self.plainPairAnalysis.setReadOnly(True)
        self.plainPairAnalysis.setBackgroundVisible(False)
        self.plainPairAnalysis.setCenterOnScroll(False)

        self.horizontalLayout_2.addWidget(self.plainPairAnalysis)


        self.verticalLayout_5.addLayout(self.horizontalLayout_2)

        self.label_10 = QLabel(Form)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setEnabled(False)

        self.verticalLayout_5.addWidget(self.label_10)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setSpacing(5)
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.horizontalLayout_4.setContentsMargins(5, 5, 5, 5)
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_4.addItem(self.horizontalSpacer)

        self.pushBack = QPushButton(Form)
        self.pushBack.setObjectName(u"pushBack")

        self.horizontalLayout_4.addWidget(self.pushBack)

        self.pushUpdate = QPushButton(Form)
        self.pushUpdate.setObjectName(u"pushUpdate")

        self.horizontalLayout_4.addWidget(self.pushUpdate)

        self.pushNext = QPushButton(Form)
        self.pushNext.setObjectName(u"pushNext")

        self.horizontalLayout_4.addWidget(self.pushNext)


        self.verticalLayout_5.addLayout(self.horizontalLayout_4)


        self.horizontalLayout.addLayout(self.verticalLayout_5)


        self.retranslateUi(Form)

        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"debyetools - [crystal structure]", None))
        self.label.setText(QCoreApplication.translate("Form", u"cell:", None))
        ___qtablewidgetitem = self.tableCell.horizontalHeaderItem(0)
        ___qtablewidgetitem.setText(QCoreApplication.translate("Form", u"New Column", None));
        ___qtablewidgetitem1 = self.tableCell.horizontalHeaderItem(1)
        ___qtablewidgetitem1.setText(QCoreApplication.translate("Form", u"New Column", None));
        ___qtablewidgetitem2 = self.tableCell.horizontalHeaderItem(2)
        ___qtablewidgetitem2.setText(QCoreApplication.translate("Form", u"New Column", None));
        ___qtablewidgetitem3 = self.tableCell.verticalHeaderItem(0)
        ___qtablewidgetitem3.setText(QCoreApplication.translate("Form", u"x", None));
        ___qtablewidgetitem4 = self.tableCell.verticalHeaderItem(1)
        ___qtablewidgetitem4.setText(QCoreApplication.translate("Form", u"y", None));
        ___qtablewidgetitem5 = self.tableCell.verticalHeaderItem(2)
        ___qtablewidgetitem5.setText(QCoreApplication.translate("Form", u"z", None));

        __sortingEnabled = self.tableCell.isSortingEnabled()
        self.tableCell.setSortingEnabled(False)
        ___qtablewidgetitem6 = self.tableCell.item(0, 0)
        ___qtablewidgetitem6.setText(QCoreApplication.translate("Form", u"4.04", None));
        ___qtablewidgetitem7 = self.tableCell.item(0, 1)
        ___qtablewidgetitem7.setText(QCoreApplication.translate("Form", u"0.0", None));
        ___qtablewidgetitem8 = self.tableCell.item(0, 2)
        ___qtablewidgetitem8.setText(QCoreApplication.translate("Form", u"0.0", None));
        ___qtablewidgetitem9 = self.tableCell.item(1, 0)
        ___qtablewidgetitem9.setText(QCoreApplication.translate("Form", u"0.0", None));
        ___qtablewidgetitem10 = self.tableCell.item(1, 1)
        ___qtablewidgetitem10.setText(QCoreApplication.translate("Form", u"4.04", None));
        ___qtablewidgetitem11 = self.tableCell.item(1, 2)
        ___qtablewidgetitem11.setText(QCoreApplication.translate("Form", u"0.0", None));
        ___qtablewidgetitem12 = self.tableCell.item(2, 0)
        ___qtablewidgetitem12.setText(QCoreApplication.translate("Form", u"0.0", None));
        ___qtablewidgetitem13 = self.tableCell.item(2, 1)
        ___qtablewidgetitem13.setText(QCoreApplication.translate("Form", u"0.0", None));
        ___qtablewidgetitem14 = self.tableCell.item(2, 2)
        ___qtablewidgetitem14.setText(QCoreApplication.translate("Form", u"4.04", None));
        self.tableCell.setSortingEnabled(__sortingEnabled)

        self.label_2.setText(QCoreApplication.translate("Form", u"basis:", None))
        ___qtablewidgetitem15 = self.tableBasis.horizontalHeaderItem(0)
        ___qtablewidgetitem15.setText(QCoreApplication.translate("Form", u"x", None));
        ___qtablewidgetitem16 = self.tableBasis.horizontalHeaderItem(1)
        ___qtablewidgetitem16.setText(QCoreApplication.translate("Form", u"y", None));
        ___qtablewidgetitem17 = self.tableBasis.horizontalHeaderItem(2)
        ___qtablewidgetitem17.setText(QCoreApplication.translate("Form", u"z", None));
        ___qtablewidgetitem18 = self.tableBasis.horizontalHeaderItem(3)
        ___qtablewidgetitem18.setText(QCoreApplication.translate("Form", u"type", None));
        ___qtablewidgetitem19 = self.tableBasis.verticalHeaderItem(0)
        ___qtablewidgetitem19.setText(QCoreApplication.translate("Form", u"atom 1", None));
        ___qtablewidgetitem20 = self.tableBasis.verticalHeaderItem(1)
        ___qtablewidgetitem20.setText(QCoreApplication.translate("Form", u"atom 2", None));
        ___qtablewidgetitem21 = self.tableBasis.verticalHeaderItem(2)
        ___qtablewidgetitem21.setText(QCoreApplication.translate("Form", u"atom 3", None));
        ___qtablewidgetitem22 = self.tableBasis.verticalHeaderItem(3)
        ___qtablewidgetitem22.setText(QCoreApplication.translate("Form", u"atom 4", None));
        ___qtablewidgetitem23 = self.tableBasis.verticalHeaderItem(4)
        ___qtablewidgetitem23.setText(QCoreApplication.translate("Form", u"atom 5", None));
        ___qtablewidgetitem24 = self.tableBasis.verticalHeaderItem(5)
        ___qtablewidgetitem24.setText(QCoreApplication.translate("Form", u"atom 6", None));
        ___qtablewidgetitem25 = self.tableBasis.verticalHeaderItem(6)
        ___qtablewidgetitem25.setText(QCoreApplication.translate("Form", u"atom 7", None));
        ___qtablewidgetitem26 = self.tableBasis.verticalHeaderItem(7)
        ___qtablewidgetitem26.setText(QCoreApplication.translate("Form", u"atom 8", None));
        ___qtablewidgetitem27 = self.tableBasis.verticalHeaderItem(8)
        ___qtablewidgetitem27.setText(QCoreApplication.translate("Form", u"atom 9", None));
        ___qtablewidgetitem28 = self.tableBasis.verticalHeaderItem(9)
        ___qtablewidgetitem28.setText(QCoreApplication.translate("Form", u"atom 10", None));

        __sortingEnabled1 = self.tableBasis.isSortingEnabled()
        self.tableBasis.setSortingEnabled(False)
        ___qtablewidgetitem29 = self.tableBasis.item(0, 0)
        ___qtablewidgetitem29.setText(QCoreApplication.translate("Form", u"0", None));
        ___qtablewidgetitem30 = self.tableBasis.item(0, 1)
        ___qtablewidgetitem30.setText(QCoreApplication.translate("Form", u"0", None));
        ___qtablewidgetitem31 = self.tableBasis.item(0, 2)
        ___qtablewidgetitem31.setText(QCoreApplication.translate("Form", u"0", None));
        ___qtablewidgetitem32 = self.tableBasis.item(1, 0)
        ___qtablewidgetitem32.setText(QCoreApplication.translate("Form", u"0.5", None));
        ___qtablewidgetitem33 = self.tableBasis.item(1, 1)
        ___qtablewidgetitem33.setText(QCoreApplication.translate("Form", u"0.5", None));
        ___qtablewidgetitem34 = self.tableBasis.item(1, 2)
        ___qtablewidgetitem34.setText(QCoreApplication.translate("Form", u"0", None));
        ___qtablewidgetitem35 = self.tableBasis.item(2, 0)
        ___qtablewidgetitem35.setText(QCoreApplication.translate("Form", u"0.5", None));
        ___qtablewidgetitem36 = self.tableBasis.item(2, 1)
        ___qtablewidgetitem36.setText(QCoreApplication.translate("Form", u"0", None));
        ___qtablewidgetitem37 = self.tableBasis.item(2, 2)
        ___qtablewidgetitem37.setText(QCoreApplication.translate("Form", u"0.5", None));
        ___qtablewidgetitem38 = self.tableBasis.item(3, 0)
        ___qtablewidgetitem38.setText(QCoreApplication.translate("Form", u"0", None));
        ___qtablewidgetitem39 = self.tableBasis.item(3, 1)
        ___qtablewidgetitem39.setText(QCoreApplication.translate("Form", u"0.5", None));
        ___qtablewidgetitem40 = self.tableBasis.item(3, 2)
        ___qtablewidgetitem40.setText(QCoreApplication.translate("Form", u"0.5", None));
        self.tableBasis.setSortingEnabled(__sortingEnabled1)

        self.label_3.setText(QCoreApplication.translate("Form", u"cutoff:", None))
        self.label_4.setText(QCoreApplication.translate("Form", u"# of nearest neighbors:", None))
        self.label_5.setText(QCoreApplication.translate("Form", u"parameters initial guess:", None))
        self.lineEditCutoff.setText("")
        self.lineEditNnn.setText("")
        self.lineEditInitialGuess.setText("")
        self.label_7.setText(QCoreApplication.translate("Form", u"[A]", None))
        self.label_6.setText("")
        self.label_8.setText("")
        self.label_10.setText("")
        self.pushBack.setText(QCoreApplication.translate("Form", u"Back", None))
        self.pushUpdate.setText(QCoreApplication.translate("Form", u"Update", None))
        self.pushNext.setText(QCoreApplication.translate("Form", u"Next", None))
    # retranslateUi




from PySide6.QtWidgets import QDialog

from debyetools.tpropsgui.atomtools import atomic_mass
from debyetools.tpropsgui.ui_dialog_periodictable import Ui_Dialog as Ui_Dialog


def create_formula_from_dict(d):
    result = ""
    for key, value in d.items():
        if value > 0:
            if value == 1:
                result += key  # Add just the key if the value is 1
            else:
                result += f"{key}{value}"  # Add key and value if value is greater than 1
    return result

class dialogPeriodicTable(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_Dialog()
        self.ui.setupUi(self)

        self.formula = ''
        self.mass = 0  # 0.02698
        self.dict_formula = {k:0 for k in [getattr(self.ui, 'pushB'+str(i)).text() for i in range(118)]}
        # self.dict_formula['Al']=0#4
        # Example setup with multiple counting buttons
        self.buttons = [getattr(self.ui, 'pushB'+str(i)) for i in range(118)]
        self.buttons[12].counter = 0  #4
        for btn in self.buttons:
            btn.counterChanged2.connect(self.updateFormula)
            # self.layout.addWidget(btn)
            btn.set_style()

        self.ui.pushButton_3.clicked.connect(self.on_pushButton_OK)





    def updateFormula(self, str, counter):
        self.dict_formula[str] = counter
        self.formula = create_formula_from_dict(self.dict_formula)

        # print('formula', self.formula)
        for btn in self.buttons:
            btn.set_style()
        self.ui.lineEdit.setText(self.formula)

        mass = 0
        total_count = 0
        for k,v in self.dict_formula.items():
            total_count += int(v)
            mass += atomic_mass[k]*int(v)

        self.mass = mass/total_count/1000


        # self.ui.browse.clicked.connect(self.getfiles)
        # self.ui.ok.clicked.connect(self.on_pushButton_OK)
        # self.filepath='.'

    # def getfiles(self):
    #         filepath = QFileDialog.getExistingDirectory(self, caption='Select a folder')
    #         self.ui.filepath.setText(filepath)

    def on_pushButton_OK(self):
        # self.filepath = self.ui.filepath.text()
        self.close()

    def closeEvent(self, event):
        # print('something')
        self.external_lineEdit.setText(self.formula)
        self.external_lineEdit2.setText(f'{self.mass}')
        event.accept()

